#!/usr/bin/env python3
# @Author   : Yang Liu
# @FileName : plot_figure.py
# @Software : NANOME project
# @Organization : JAX Li Lab
# @Website  : https://github.com/TheJacksonLaboratory/nanome

"""
Plots and export data for nanome paper
"""
import argparse
import glob
import gzip
import os.path
import pickle
import re
from collections import defaultdict

import matplotlib.pyplot as plt
#### Additional libraries:
import numpy as np
import pandas as pd
import pybedtools
import seaborn as sns
from scipy import stats
from scipy.stats import pearsonr
from sklearn import metrics
from sklearn.metrics import precision_recall_curve

from nanocompare.eval_common import logger, set_log_debug_level, pic_base_dir
from nanocompare.global_settings import *


def set_style(font_scale=1.2):
    # sns.set()
    # sns.set(font_scale=1.15)
    # sns.set_style("white")

    # This sets reasonable defaults for font size for
    # a figure that will go in a paper
    sns.set_context("paper")
    sns.set(font_scale=font_scale)

    # Set the font to be serif, rather than sans
    # sns.set(font='serif')

    # Make the background white, and specify the
    # specific font family
    # sns.set_style("white", {
    #         "font.family": "serif",
    #         "font.serif" : ["Times", "Palatino", "serif"]
    #         })
    sns.set_style("white")


def grid_plot_correlation_matrix_for_fig5a(infn, removeDeepMod=False):
    """
    Plot the grid of corr COE, distribution and scatter plot based on input files
    :return:
    """
    ## Load data into df
    logger.debug(infn)
    df = pd.read_csv(infn, sep=',')

    sel_col = []
    rename_dict = {}
    for col in df.columns:
        if str(col).endswith('_freq'):
            sel_col.append(col)

            new_col_name = get_tool_name(str(col).replace('_freq', ''))
            rename_dict.update({str(col): new_col_name})
    df = df[sel_col]
    df = df.rename(columns=rename_dict)

    if removeDeepMod:
        # rename DeepMod column here
        df = df.drop("DeepMod", axis=1)

    num_col = len(df.columns)

    df = df.iloc[:, list(range(1, num_col)) + [0]]
    logger.debug(f'Load data = {len(df)}')

    # Rearrange columns by correlation from high to low
    corrdf = df.corr()
    # logger.info(corrdf)

    coe_series = corrdf.iloc[:, -1]
    # logger.info(coe_series[:-1].sort_values(ascending=False))

    orderedColumns = coe_series[:-1].sort_values(ascending=False).index.tolist()

    df = df.loc[:, ['BGTruth'] + orderedColumns]

    # Rename to BS-seq name based on WR's suggestions
    df = df.rename(columns={'BGTruth': 'BS-seq'})

    ## Plot correlation grid figure
    basefn = os.path.basename(infn)
    outfileName = "{}.jpg".format("fig.5a.s5abc." + basefn.replace(".csv", ""))

    if args.o:
        outdir = args.o
    else:
        outdir = pic_base_dir

    outfn = os.path.join(outdir, outfileName)

    plt.clf()

    fig, ax = plt.subplots()

    fig.set_size_inches(12, 12)

    left, width = .25, .5
    bottom, height = .25, .5
    right = left + width
    top = bottom + height

    position = 1
    for yrow in range(1, num_col + 1):  # row
        for xcol in range(1, num_col + 1):  # column, each tool
            # adding this so that one would be able to estimate how much time one has for a coffe
            logger.debug(f"Please wait, processing column xcol {xcol}, yrow {yrow}.")
            if xcol == yrow:
                # Diagonal, distribution lines:
                plt.subplot(num_col, num_col, position)
                params = {'legend.fontsize': 50,
                          'legend.handlelength': 0,
                          'legend.handletextpad': 0,
                          'legend.fancybox': False,
                          'legend.loc': 'upper right',
                          'legend.framealpha': 0,
                          'legend.borderaxespad': 0}
                plt.rcParams.update(params)
                ax = sns.kdeplot(df.iloc[:, xcol - 1], shade=False, color="black", legend=True)

                tool_fontsize = 10
                tool_fontsize = 14
                leg = ax.legend(labels=[str(df.columns[xcol - 1])], fontsize=tool_fontsize)
                for item in leg.legendHandles:
                    item.set_visible(False)

                # ax.annotate("abcd", xy=(0.2, 0.8), xycoords='axes fraction',
                #             fontsize=14)
                ax.set_yticklabels([])
                ax.set_xticklabels([])
                ax.set_ylabel('')
                ax.set_xlabel('')
                ax.tick_params(axis=u'both', which=u'both', length=0)
            elif xcol > yrow:
                # upper triangle, COE number:
                ax2 = plt.subplot(num_col, num_col, position)

                corrValue = pearsonr(df.iloc[:, xcol - 1], df.iloc[:, yrow - 1])
                corrValueStr = "{0:.3f}".format(corrValue[0])

                coe_font_size = 19
                coe_font_size = 22

                if yrow == 1:
                    ax2.text(0.5 * (left + right), 0.5 * (bottom + top), corrValueStr,
                             horizontalalignment='center',
                             verticalalignment='center',
                             fontsize=coe_font_size, color='#7B2323', weight='bold',
                             transform=ax2.transAxes)
                else:
                    ax2.text(0.5 * (left + right), 0.5 * (bottom + top), corrValueStr,
                             horizontalalignment='center',
                             verticalalignment='center',
                             fontsize=coe_font_size, color='black',
                             transform=ax2.transAxes)

                ax2.set_yticklabels([])
                ax2.set_xticklabels([])
                ax2.tick_params(axis=u'both', which=u'both', length=0)

            elif xcol < yrow:
                # lower triangle, scatter plot, using hexbin, x-column label, y-row label:
                ax3 = plt.subplot(num_col, num_col, position)

                ### Kernel density plot:
                BGres = 300j  # 150j # 75j # 300j ## for debugging change this parameter to "50j" or less

                m1_sign = np.array(list(df.iloc[:, xcol - 1]))
                m2_sign = np.array(list(df.iloc[:, yrow - 1]))
                xmin = ymin = -0.05
                xmax = ymax = 1.05

                X, Y = np.mgrid[xmin:xmax:BGres, ymin:ymax:BGres]
                positions = np.vstack([X.ravel(), Y.ravel()])
                values = np.vstack([m1_sign, m2_sign])
                kernel = stats.gaussian_kde(values)
                Z = np.reshape(kernel(positions).T, X.shape)

                ax3.imshow(np.rot90(Z), cmap=plt.cm.Spectral_r, extent=[xmin, xmax, ymin, ymax], aspect="auto")
                # ax3.imshow(np.rot90(Z), cmap=plt.cm.gist_stern, extent=[xmin, xmax, ymin, ymax], aspect="auto") # An example how to switch from Spectral_r to gist_stern colormap
                # ax3.plot(m1_sign, m2_sign, marker=',', color="white", linestyle='None', markersize=0.25, alpha=0.25) # uncomment this line if you want to generate version with points marker very lightly on the figure

                ### Scatter plot (previous implementation for the plot - i am keeping this for now but should be cleaned if we decide to go with KDE version)
                # if len(df) < 100:
                #                     gridRes = 15  # for HL60
                #                 else:
                #                     gridRes = 50
                #
                #                 mincnt = 1
                #
                #                 if len(df) < 100:  # for HL60 plot
                #                     plt.hexbin(df.iloc[:, xcol - 1], df.iloc[:, yrow - 1], gridsize=(gridRes, gridRes), cmap='Blues', bins='log', mincnt=None)
                #                     pass
                #                 else:
                #                     plt.hexbin(df.iloc[:, xcol - 1], df.iloc[:, yrow - 1], gridsize=(gridRes, gridRes), cmap='Blues', bins='log', mincnt=mincnt)
                tick_label_fontsize = 10
                tick_label_fontsize = 12
                if yrow == num_col:  # last row scatter plot shows x ticks
                    plt.xticks([0, 1], fontsize=tick_label_fontsize)
                    ax3.set_xticklabels(["0%", "100%"], fontsize=tick_label_fontsize)
                    for label, alnType in zip(ax3.get_xticklabels(), ['left', 'center']):
                        label.set_horizontalalignment(alnType)
                else:
                    plt.xticks([], fontsize=10)

                if xcol == 1:  # first column scatter plot shows y ticks
                    plt.yticks([0, 1], fontsize=tick_label_fontsize)
                    ax3.set_yticklabels(["0%", "100%"], fontsize=tick_label_fontsize)
                    for label, alnType in zip(ax3.get_yticklabels(), ['bottom', 'top']):
                        label.set_verticalalignment(alnType)
                else:
                    plt.yticks([], fontsize=10)

            position += 1

    fig.savefig(outfn, dpi=300, bbox_inches='tight')
    fig.savefig(outfn.replace(".jpg", ".pdf"), dpi=300, bbox_inches='tight')  # Generate also PDF version
    # plt.show()
    plt.close()
    logger.info(f"save to {outfn}")


def gen_figure_5a(infn):
    """
    Generate figure 5-a in paper
    Correlation plots
    :return:
    """
    grid_plot_correlation_matrix_for_fig5a(infn)
    pass


def plot_ROC_PR_curves(ret, outdir, tagname="tagname", removeDeepMod=False):
    """
    Plot ROC AUC and PR curves
    :param ret:
    :param outdir:
    :param tagname:
    :return:
    """
    figure_size = (4, 4)

    # title_font_size = 18
    label_font_size = 14

    ## Plot ROC curve
    plt.clf()
    plt.figure(figsize=figure_size)

    csfont = {'fontsize': label_font_size}

    conf_matrix = {}
    for toolname, toolcolor in zip(ToolNameList, ToolsColorList):
        if toolname == 'DeepMod' and removeDeepMod:
            continue
        ytrue = ret[f'{toolname}_true']
        ypred = ret[f'{toolname}_pred']
        yscore = ret[f'{toolname}_score']

        ## Construct comfusion matrix
        from sklearn.metrics import confusion_matrix
        conf_matrix[toolname] = confusion_matrix(ytrue, ypred, labels=[0, 1])

        # logger.debug(f'ytrue={len(ytrue)}, ypred={len(ypred)}, yscore={len(yscore)}')
        # TODO: plot points in curves
        fpr, tpr, threshold = metrics.roc_curve(ytrue, yscore, drop_intermediate=False)
        roc_auc = metrics.auc(fpr, tpr)
        plt.plot(fpr, tpr, toolcolor, label=f'{toolname}={roc_auc:.2f}')
    leg = plt.legend(loc='lower right', title="AUC")
    leg._legend_box.align = "left"
    # plt.plot([0, 1], [0, 1], 'r--')
    plt.xlim([0, 1])
    plt.ylim([0, 1])

    plt.ylabel('True Positive Rate', **csfont)
    plt.xlabel('False Positive Rate', **csfont)

    # plt.title('ROC Curves', fontsize=title_font_size)
    outfn = os.path.join(outdir, f'fig.3b.{tagname}.roc.curves.jpg')
    plt.savefig(outfn, format='jpg', bbox_inches='tight', dpi=300)
    plt.savefig(outfn.replace(".jpg", ".pdf"), dpi=300, bbox_inches='tight')  # Generate also PDF version

    # print(plt.rcParams["font.family"])
    # print(plt.rcParams['font.sans-serif'])

    # plt.show()
    plt.close()

    outfn = os.path.join(outdir, f'confusion.matrix.{tagname}.each.tool.csv.gz')
    outf = gzip.open(outfn, 'wt')
    for toolname in ToolNameList:
        if toolname not in conf_matrix:
            continue
        # logger.info(f"{toolname} = {conf_matrix[toolname]}")
        outf.write(f"{toolname}\n")
        mat = conf_matrix[toolname]
        outf.write(f"{mat[0, 0]},{mat[0, 1]}\n")
        outf.write(f"{mat[1, 0]},{mat[1, 1]}\n")
    outf.close()
    return

    ## PR curves
    plt.clf()
    plt.figure(figsize=figure_size)
    for toolname, toolcolor in zip(ToolNameList, ToolsColorList):
        if toolname == 'DeepMod' and removeDeepMod:
            continue
        ytrue = ret[f'{toolname}_true']
        yscore = ret[f'{toolname}_score']

        precision, recall, _ = precision_recall_curve(ytrue, yscore)
        average_precision = metrics.average_precision_score(ytrue, yscore)
        plt.plot(recall, precision, toolcolor, label=f'{toolname}={average_precision:.2f}')
    plt.legend(loc='lower left')
    plt.xlim([0, 1])
    plt.xlabel('Recall', fontsize=label_font_size)
    plt.ylabel('Precision', fontsize=label_font_size)
    plt.ylim([0.0, 1.05])
    # plt.title('Precision-Recall Curves', fontsize=title_font_size)

    outfn = os.path.join(outdir, f'fig.3b.{tagname}.pr.curves.jpg')
    plt.savefig(outfn, format='png', bbox_inches='tight', dpi=600)

    plt.show()
    plt.close()


def collect_singleton_vs_nonsingleton_df(runPrefix, pattern="*.summary.bsseq.singleton.nonsingleton.cov1.csv"):
    dflist = []
    # logger.debug(runPrefix)
    for run1 in runPrefix:
        filepat = os.path.join(runPrefix[run1], pattern)
        # logger.debug(run1)
        flist = glob.glob(filepat)
        if len(flist) != 1:
            logger.error(
                f"Too much/No summary of singleton vs non-singleton for {run1} in folder {runPrefix[run1]} with pattern={pattern}, len={len(flist)}")
        # logger.debug(f'Get file:{flist[0]}')
        if len(flist) >= 1:
            logger.info(f"read file: {flist[0]}")
            df = pd.read_csv(flist[0], index_col=0)
            dflist.append(df)
    retdf = pd.concat(dflist)
    retdf.index.name = 'Dataset'
    return retdf


def collect_distribution_genomic_df(runPrefix, pattern=None):
    dflist = []
    logger.debug(runPrefix)
    for run1 in runPrefix:
        filepat = os.path.join(runPrefix[run1], pattern)
        # logger.debug(run1)
        flist = glob.glob(filepat)
        if len(flist) != 1:
            logger.error(
                f"Too much/No summary of singleton vs non-singleton for {run1} in folder {runPrefix[run1]} with pattern={pattern}, len={len(flist)}")

        # logger.debug(f'Get file:{flist[0]}')
        if len(flist) >= 1:
            logger.info(f"read file: {flist[0]}")
            df = pd.read_excel(flist[0], index_col=0, engine='openpyxl')  #
            dflist.append(df)
    retdf = pd.concat(dflist)

    retdf['Dataset'] = pd.Categorical(retdf['Dataset'], datasets_order)
    retdf['Coord'] = pd.Categorical(retdf['Coord'], locations_order)
    retdf = retdf.sort_values(by=['Dataset', 'Coord'], ascending=[True, True])
    retdf = retdf.dropna()

    return retdf


def collect_performance_report_as_df(runPrefix):
    """
    create report from list of runPrefix, return specified columns
    :return:
    """
    dflist = []
    for runKey in runPrefix:
        logger.debug(f'runPrefix={runKey}')
        pattern = os.path.join(runPrefix[runKey], "performance?results", "*.performance.report.csv")
        files = glob.glob(pattern)
        for infn in files:
            df = pd.read_csv(infn, index_col=0, sep=",")
            dflist.append(df)
            logger.debug(f'Collect data from {runKey}:{os.path.basename(infn)} = {len(df)}')
    if len(dflist) == 0:
        raise Exception(f"Can not find report at {runKey} in folder {runPrefix[runKey]} with pattern={pattern}")
    combdf = pd.concat(dflist, ignore_index=True)
    logger.info(f'We collected total {len(dflist)} files with {len(combdf)} records')
    return combdf


def load_wide_format_performance_results(runPrefix,
                                         sel_locations=locations_category + locations_singleton + locations_new):
    """
    Collect the currently new performance of exp results for paper
    :return:
    """
    df = collect_performance_report_as_df(runPrefix)
    retdf = df[df['Location'].isin(sel_locations)]

    logger.debug(
        f"load_wide_format_performance_results, wide-format retdf={len(retdf)} using locations={sel_locations}")

    return retdf


def save_wide_format_performance_results(runPrefix, outdir, tagname):
    """
    Save all performance report results into a csv
    :return:
    """
    df = load_wide_format_performance_results(runPrefix)

    ## Rename Tool name to starndard name
    df["Tool"].replace({"Megalodon_ZW": "Megalodon", "DeepMod_C": "DeepMod"}, inplace=True)

    ## arrange df
    df['Dataset'] = pd.Categorical(df['Dataset'], datasets_order)
    df['Tool'] = pd.Categorical(df['Tool'], ToolNameList)
    df['Location'] = pd.Categorical(df['Location'], locations_order)
    df = df.sort_values(by=['Dataset', 'Location', 'Accuracy'], ascending=[True, True, False])

    outfn = os.path.join(outdir, f'performance-results{f"-{tagname}" if tagname else ""}.csv')
    df.to_csv(outfn)
    logger.info(f'save to {outfn}')


def parse_arguments():
    parser = argparse.ArgumentParser(prog='plot_figure (NANOME)', description='Plot and export data for Nanocompare paper.')
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s v{nanome_version}')
    parser.add_argument("cmd", help="name of command, fig5a, export-data, etc.")
    parser.add_argument('-i', nargs='+', help='list of input files', default=[])
    parser.add_argument('-o', type=str, help="output dir",
                        default=None)  # TODO: check all correct when change this to None
    parser.add_argument('--tagname', type=str, help="tagname of files", default=None)
    parser.add_argument('--beddir', type=str, help="bed file dir used for finding Concordant and Discordant",
                        default=None)
    parser.add_argument('--cutoff', type=int, help="the cutoff used on bed file", default=1)
    parser.add_argument('--signal-col', type=int, help="the column number of signals extract from bed to bedGraph",
                        default=6)
    parser.add_argument('--cov-col', type=int, help="the column number of signals extract from bed to bedGraph",
                        default=7)  # -1 means no cov col

    args = parser.parse_args()
    return args


if __name__ == '__main__':
    set_log_debug_level()

    ## Set tmp dir for bedtools
    bedtool_tmp_dir = "/fastscratch/liuya/nanocompare/bedtools_tmp"
    os.makedirs(bedtool_tmp_dir, exist_ok=True)
    pybedtools.helpers.set_tempdir(bedtool_tmp_dir)

    ## Use same font Arial for all plotting
    import matplotlib.font_manager as font_manager

    # font_manager._rebuild() # some times failed, must be sure change font
    plt.rcParams['font.sans-serif'] = ['Arial']

    args = parse_arguments()
    logger.debug(args)

    if args.cmd == 'fig5a':
        ## find ${resultDir} -name 'Meth_corr_plot_data*.csv' \
        ##      -type f -exec python plot_figure.py fig5a -i {} \;
        ## python plot_figure.py fig5a -i /projects/li-lab/Nanopore_compare/result/MethCorr-NA19240_RRBS/Meth_corr_plot_data_joined-NA19240_RRBS-bsCov5-minToolCov3-baseFormat1.csv

        if not args.o:
            outdir = pic_base_dir
        for fn in args.i:
            gen_figure_5a(fn)
    elif args.cmd == 'figs5ab-data':
        # Fig. S5A-B data: PCC in each regions
        if not args.o:
            args.o = pic_base_dir
        dflist = []
        for indir in args.i:
            # file like: NA12878.corrdata.coe.pvalue.each.regions.xlsx
            fnlist = glob.glob(os.path.join(indir, '*.corrdata.coe.pvalue.each.regions.xlsx'))
            if len(fnlist) != 1:
                raise Exception(f'Found more or none fnlist={fnlist}, for indir={indir}')
            logger.info(f'Find file: {fnlist[0]}')

            df = pd.read_excel(fnlist[0], index_col=0, engine='openpyxl')
            dflist.append(df)
        outdf = pd.concat(dflist)
        outfn = os.path.join(args.o,
                             f'All.corrdata.coe.pvalue.each.regions{f".{args.tagname}" if args.tagname else ""}.xlsx')
        outdf.to_excel(outfn)
        logger.info(f'save to {outfn}')
    elif args.cmd == 'export-data':
        # Table S3 data
        # using global setting variable runPrefixDict to locate all data
        ## python plot_figure.py export-data -i /projects/li-lab/Nanopore_compare/result/MethPerf-APL_RRBS_CPG /projects/li-lab/Nanopore_compare/result/MethPerf-HL60_RRBS_CPG /projects/li-lab/Nanopore_compare/result/MethPerf-K562_WGBS_CPG --tagname CPG

        ## python plot_figure.py export-data -i /projects/li-lab/yang/results/2021-02-22/Meth
        if len(args.i) == 0:
            raise Exception(f'No run dir specified')
        else:
            run_prefix = defaultdict()
            for dirname in args.i:
                run_prefix[os.path.basename(dirname)] = dirname
        if not args.o:
            args.o = pic_base_dir

        logger.info(run_prefix)
        save_wide_format_performance_results(run_prefix, args.o, args.tagname)

        # pattern1 = "*.summary.bsseq.singleton.nonsingleton.cov1.csv"
        # df = collect_singleton_vs_nonsingleton_df(run_prefix, pattern=pattern1)
        # ## arrange df
        # df = df.reindex(datasets_order)
        #
        # outfn = os.path.join(args.o,
        #                      f'dataset.singleton.vs.non-singleton{f".{args.tagname}" if args.tagname else ""}.cov1.csv')
        # df.to_csv(outfn)

        pattern2 = "*.summary.bsseq.singleton.nonsingleton.cov5.table.s2.csv"
        df = collect_singleton_vs_nonsingleton_df(run_prefix, pattern=pattern2)
        ## arrange df
        df = df.reindex(datasets_order)

        outfn = os.path.join(args.o,
                             f'dataset.singleton.vs.non-singleton{f".{args.tagname}" if args.tagname else ""}.cov5.table.s2.xlsx')
        df.to_excel(outfn)
        logger.info(f'save stats of singleton and non-singleton to {outfn}')

        ## concat all singleton/nonsingleton for each datasets
        ## sample: HL60.bgtruth.certain.sites.distribution.sing.nonsing.each.genomic.cov5.xlsx
        pattern3 = "*.bgtruth.certain.sites.distribution.sing.nonsing.each.genomic.cov5.table.s6.xlsx"
        df = collect_distribution_genomic_df(run_prefix, pattern=pattern3)
        ## arrange df
        # df['Dataset'] = pd.Categorical(df['Dataset'], datasets_order)
        # df['Coord'] = pd.Categorical(df['Coord'], locations_order)
        # df = df.sort_values(by='Dataset')

        outfn = os.path.join(args.o,
                             f'all.certain.sites.distribution.each.genomic.region{f".{args.tagname}" if args.tagname else ""}.cov5.table.s6.csv')
        df.to_csv(outfn)

        logger.info(f'save distribution of each genomic region to {outfn}')
    elif args.cmd == 'export-curve-data':
        # Figure 3B data
        ## python plot_figure.py export-curve-data -i /projects/li-lab/Nanopore_compare/result/MethPerf-HL60_RRBS /projects/li-lab/Nanopore_compare/result/MethPerf-K562_WGBS

        ## python plot_figure.py export-curve-data -i /projects/li-lab/Nanopore_compare/result/MethPerf-APL_RRBS_CPG /projects/li-lab/Nanopore_compare/result/MethPerf-HL60_RRBS_CPG /projects/li-lab/Nanopore_compare/result/MethPerf-K562_WGBS_CPG --tagname CPG
        if not args.o:
            args.o = pic_base_dir
        outdir = os.path.join(args.o, f'plot-curve-data{f"-{args.tagname}" if args.tagname else ""}')
        os.makedirs(outdir, exist_ok=True)

        if len(args.i) == 0:
            raise Exception("No run dir specified")

        for bdir in args.i:
            runPrefix = os.path.basename(bdir)
            cnt = 0
            for coordinate_bed_name in location_filename_to_abbvname.keys():
                curve_data = defaultdict(list)
                success = True
                for toolname in ToolNameList:
                    pattern_str = os.path.join(bdir, 'performance?results', 'curve_data',
                                               f'*.{toolname}*.*{coordinate_bed_name}.curve_data.pkl')
                    fnlist = glob.glob(pattern_str)
                    if len(fnlist) != 1:
                        logger.error(
                            f'Can not locate curve_data for Tool={toolname}, at Coord={coordinate_bed_name}, with pattern={pattern_str}, find results={fnlist}. Please check if MethPerf results folder={bdir} specified is correct.')
                        success = False
                        break
                    cnt += 1
                    with open(fnlist[0], 'rb') as f:
                        ret = pickle.load(f)
                        # logger.debug(ret)
                        curve_data[f'{toolname}_true'] = ret['yTrue']  # Ground truth label
                        curve_data[f'{toolname}_pred'] = ret['yPred']  # Prediction label
                        curve_data[f'{toolname}_score'] = ret['yScore']  # Prediction score, used for roc curve plotting
                if success:
                    outfn = os.path.join(outdir,
                                         f'{runPrefix}.plot.curve.data.ytrue.ypred.yscore.{location_filename_to_abbvname[coordinate_bed_name]}.pkl')
                    with open(outfn, 'wb') as f:
                        pickle.dump(curve_data, f)

                    outfn = os.path.join(outdir,
                                         f'{runPrefix}.plot.curve.data.ytrue.ypred.yscore.{location_filename_to_abbvname[coordinate_bed_name]}.dat')
                    with open(outfn, "w") as f:
                        for toolname in ToolNameList:
                            f.write(f"{toolname}_true:")
                            outstr = ','.join([str(value) for value in curve_data[f'{toolname}_true']])
                            f.write(outstr)
                            f.write("\n")

                            f.write(f"{toolname}_pred:")
                            outstr = ','.join([str(value) for value in curve_data[f'{toolname}_pred']])
                            f.write(outstr)
                            f.write("\n")

                            f.write(f"{toolname}_score:")
                            outstr = ','.join([f'{value:.4f}' for value in curve_data[f'{toolname}_score']])
                            f.write(outstr)
                            f.write("\n")
            logger.info(f'For runPrefix={runPrefix} at dir={bdir}, total files={cnt}')
    elif args.cmd == 'plot-curve-data':
        # Fig. 3B
        ## find /projects/li-lab/Nanopore_compare/result/plot-curve-data -name '*.pkl' -exec python plot_figure.py plot-curve-data -i {} \;
        if not args.o:
            args.o = pic_base_dir

        outdir = os.path.join(args.o, f'curves-figures{f"-{args.tagname}" if args.tagname else ""}')
        os.makedirs(outdir, exist_ok=True)
        for fn in args.i:
            logger.debug(f'Plot data from fn={fn}')
            with open(fn, 'rb') as infn:
                basename = os.path.basename(fn)
                bn = os.path.splitext(basename)[0]
                ret = pickle.load(infn)
                # logger.debug(ret.keys())
                plot_ROC_PR_curves(ret, outdir, tagname=bn)
    elif args.cmd == 'guppy-qos':
        ## collect basecall output for summary results, used for qos
        ## python plot_figure.py guppy-qos -i /fastscratch/liuya/nanocompare/HL60-Runs/HL60-N50-basecall /fastscratch/liuya/nanocompare/K562-Runs/K562-N50-basecall /projects/li-lab/Nanopore_compare/nanopore_fast5/NA19240-N300-basecall /fastscratch/liuya/nanocompare/APL-Runs/APL-N50-basecall
        for basedir in args.i:
            fpatstr = os.path.join(basedir, '*', 'sequencing_summary.txt')
            flist = glob.glob(fpatstr)
            # logger.debug(flist)
            dflist = []
            for fn in flist:
                df = pd.read_csv(fn, sep='\t')
                dflist.append(df)
            outdf = pd.concat(dflist)
            logger.info(outdf)

            bfn = os.path.basename(basedir)
            outfn = os.path.join(pic_base_dir, f'{bfn}.sequencing_summary.txt')
            outdf.to_csv(outfn, index=False, sep='\t')
    elif args.cmd == 'bed-to-bedGraph':
        # Convert bed into bedGraph, extract specific column signals (5mC, or 5hmC)
        # Input is 0-based, output is same 0-based for TSS analysis
        # python plot_figure.py bed-to-bedGraph -i $fn --cutoff $CUTOFF -o $outdir --cov_col 7  --signal-col 6
        # python plot_figure.py bed-to-bedGraph -i $fn --cutoff 5 --cov_col -1  --signal-col 3
        ## Default: cov_col=7, signal_col=6
        infn = args.i[0]
        cutoff = args.cutoff
        df = pd.read_csv(infn, sep='\t', header=None)
        logger.info(len(df))

        if args.cov_col != -1:  # filter with coverage
            df = df[df.iloc[:, args.cov_col] >= cutoff]
            logger.info(f'After cutoff={cutoff}, len(df)={len(df)}')

        if not args.o:  # output dir params
            outdir = pic_base_dir
        else:
            outdir = args.o

        baseInfn = os.path.basename(infn)
        baseOutfn = re.sub('cov..bed.gz', '', baseInfn) + f"cov{cutoff}.bedGraph"

        outfn = os.path.join(outdir, baseOutfn)
        df = df.iloc[:, [0, 1, 2, args.signal_col]]
        df.to_csv(outfn, sep='\t', header=False, index=None)
        logger.info(f'save to {outfn}')

        from pybedtools import BedTool

        bed1 = BedTool(outfn)
        bed1 = bed1.sort()
        outfn2 = outfn.replace('.bedGraph', '.sorted.bedGraph')
        bed1.saveas(outfn2)
        logger.info(f'after sort bed, save to {outfn2}')
    elif args.cmd == 'view-outmatrix':  # View the out marix for TSS, such as /projects/li-lab/yang/results/2021-04-25/tss-plots/
        ### python plot_figure.py view-outmatrix -i /projects/li-lab/yang/results/2021-04-25/tss-plots/NA19240.bin50.outMatrix.tsv
        infn = args.i[0]
        df = pd.read_csv(infn, sep='\t', compression='gzip', header=None, skiprows=[0], na_values='nan')
        logger.info(df)

        if args.o:
            outdir = args.o
        else:
            outdir = pic_base_dir
        outfn = os.path.basename(infn).replace('outMatrix.tsv', 'outMatrix.for.R.csv.gz')
        outfn = os.path.join(outdir, outfn)

        df.to_csv(outfn, index=False, header=False, sep=',', compression='gzip')
        logger.info(f'save to {outfn}')

        pass
    else:
        raise Exception(f'Command={args.cmd} is not support')

    logger.info('Plot data script DONE')



#!/usr/bin/env python3

from sys import argv

import numpy as np
from tombo import tombo_helper, tombo_stats

# python ../Tombo_extract_per_read_stats.py /projects/li-lab/NanoporeData/WR_ONT_analyses/NanoCompare/hg38.chrom.sizes TomboTest_001.batch_0.5mC.tombo.per_read_stats test2.bed


"""
Ouput format:

head K562-Tombo-N100/K562-Tombo-N100-meth-call/K562-Tombo-N100.batch_98.perReadsStats.bed
chr14	19118837	19118837	18eacd99-d2b9-48d8-a16c-cf28b351c66b	2.610044548594933	+
chr14	19118838	19118838	18eacd99-d2b9-48d8-a16c-cf28b351c66b	2.7027532865530004	+
chr14	19118852	19118852	18eacd99-d2b9-48d8-a16c-cf28b351c66b	5.296365664897572	+
"""

chromSizesInfile = argv[1]  # "/projects/li-lab/NanoporeData/WR_ONT_analyses/NanoCompare/hg38.chrom.sizes"
perReadStatsInfile = argv[2]  # "TomboTest_001.batch_0.5mC.tombo.per_read_stats"
outfileName = argv[3]  # "test.bed"


def parseChromSizesFile(infileName):
    result = {}
    infile = open(infileName, 'r')
    for row in infile:
        tmp = row.strip().split("\t")
        result[tmp[0]] = int(tmp[1])
    infile.close()
    return result


chromSizes = parseChromSizesFile(chromSizesInfile)
outfile = open(outfileName, "w")

per_read_stats = tombo_stats.PerReadStats(perReadStatsInfile)

for chrm in chromSizes:
    ####save plus strand
    int_data = tombo_helper.intervalData(chrm=chrm, start=1, end=chromSizes[chrm], strand='+')
    reg_per_read_stats_plus = per_read_stats.get_region_per_read_stats(int_data)
    if isinstance(reg_per_read_stats_plus, np.ndarray):
        """
        Structure of each cpg is as:
         (50214, 2.95450765, "b'3526811b-6958-49f8-b78c-a205c1b5fc6e'")
        """
        for cpg in reg_per_read_stats_plus:
            outfile.write("{}\t{}\t{}\t{}\t{}\t{}\n".format(chrm, cpg[0], cpg[0], cpg[2][2:-1], cpg[1], "+"))

    ####save minus strand
    int_data = tombo_helper.intervalData(chrm=chrm, start=1, end=chromSizes[chrm], strand='-')
    reg_per_read_stats_minus = per_read_stats.get_region_per_read_stats(int_data)
    if isinstance(reg_per_read_stats_plus, np.ndarray):
        for cpg in reg_per_read_stats_plus:
            outfile.write("{}\t{}\t{}\t{}\t{}\t{}\n".format(chrm, cpg[0], cpg[0], cpg[2][2:-1], cpg[1], "-"))

outfile.close()



import os.path
from collections import defaultdict

import pandas as pd

from nanocompare.eval_common import import_call
from nanocompare.global_config import set_log_debug_level, logger, pic_base_dir


def combine_both_strand_by_meteore(call_dict):
    """
    Combine both strand as METEORE paper
    :param call_dict:
    :return:
    """
    combine_dict = defaultdict(tuple)

    for key in call_dict:
        (chr, start, strand) = key
        if strand == '+':
            pos_key = (chr, start, '+')
            neg_key = (chr, start + 1, '-')
            combine_key = (chr, start)
        elif strand == '-':
            pos_key = (chr, start - 1, '+')
            neg_key = (chr, start, '-')
            combine_key = (chr, start - 1)
        else:
            raise Exception(f"Not correct key={key}")

        if combine_key in combine_dict:
            continue
        ## Combine both strand, and average the methylation freqency

        combine_list = list()
        if pos_key in call_dict and neg_key in call_dict:
            pos_cov = len(call_dict[pos_key])
            pos_freq = sum(call_dict[pos_key]) / len(call_dict[pos_key])
            neg_cov = len(call_dict[neg_key])
            neg_freq = sum(call_dict[neg_key]) / len(call_dict[neg_key])
            ## Ref: https://github.com/comprna/METEORE/blob/317484e6ae33c3bfc6d84f83dd73d1863965d483/script/run_deepmod.R#L46
            coverage = pos_cov + neg_cov
            meth_freq = 0.5 * (pos_freq + neg_freq)
        elif pos_key in call_dict:
            combine_list += list(call_dict[pos_key])
            coverage = len(call_dict[pos_key])
            meth_freq = sum(call_dict[pos_key]) / len(call_dict[pos_key])
        elif neg_key in call_dict:
            combine_list += list(call_dict[neg_key])
            coverage = len(call_dict[neg_key])
            meth_freq = sum(call_dict[neg_key]) / len(call_dict[neg_key])
        # meth_freq = sum(combine_list) / len(combine_list)
        # coverage = len(combine_list)
        combine_dict[combine_key] = (meth_freq, coverage)
        # logger.debug(f"combine_key={combine_key}, meth_freq={meth_freq}, coverage={coverage}")
    return combine_dict


def get_keys_from_meteore():
    """
    Using chr pos in METEORE paper for sanity check, we are sure it is 1-based in pos
    :param df:
    :return:
    """
    infn_meteore_supp_data11 = '/fastscratch/liuya/nanocompare/deepmod_problem/Supplementary_data_11.xlsx'
    df_meteore = pd.read_excel(infn_meteore_supp_data11, engine='openpyxl')
    logger.info(df_meteore)

    ret_keys = list()
    for index, row in df_meteore.iterrows():
        ret_keys += [(row['chr'], int(row['pos']))]
    return ret_keys


if __name__ == '__main__':
    set_log_debug_level()

    selected_keys = get_keys_from_meteore()

    infn_deepmod = '/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs/NA12878.deepmod.C.combine_allchrs.bed.gz'
    infn_nanopolish = '/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs/NA12878.nanopolish.methylation_calls.combine_allchrs.tsv.gz'
    infn_meteore = '/projects/li-lab/Nanopore_compare/suppdata/METEORE_results/NA12878.METEORE.megalodon_deepsignal-optimized-model-perRead.combine.tsv.gz'
    infn_megalodon = '/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs/NA12878.megalodon.per_read.combine_allchrs.bed.gz'
    infn_deepsignal = '/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs/NA12878.deepsignal.call_mods.combine_allchrs.tsv.gz'
    infn_tombo = '/projects/li-lab/Nanopore_compare/data/NA12878/combine_allchrs/NA12878.tombo.perReadsStats.combine_allchrs.bed.gz'

    infn_bg1 = '/projects/li-lab/Nanopore_compare/data/NA12878/ENCFF279HCL.bed.gz'
    infn_bg2 = '/projects/li-lab/Nanopore_compare/data/NA12878/ENCFF835NTC.bed.gz'

    baseFormat = 1
    using_cache = True
    enable_cache = True
    # using_cache = False
    # enable_cache = False

    ## Get input as key-> value: key=(chr, pos, strand), value = [0011000]
    dict_meteore = import_call(infn_meteore, "METEORE", baseFormat=baseFormat, include_score=False, siteLevel=False,
                               using_cache=using_cache, enable_cache=enable_cache)
    dict2_meteore = combine_both_strand_by_meteore(dict_meteore)

    dict_deepmod = import_call(infn_deepmod, "DeepMod.C", baseFormat=baseFormat, include_score=False, siteLevel=False,
                               using_cache=using_cache, enable_cache=enable_cache)
    dict2_deepmod = combine_both_strand_by_meteore(dict_deepmod)

    dict_nanopolish = import_call(infn_nanopolish, "Nanopolish", baseFormat=baseFormat, include_score=False,
                                  siteLevel=False,
                                  using_cache=using_cache, enable_cache=enable_cache)
    dict2_nanopolish = combine_both_strand_by_meteore(dict_nanopolish)

    dict_megalodon = import_call(infn_megalodon, "Megalodon", baseFormat=baseFormat, include_score=False,
                                 siteLevel=False,
                                 using_cache=using_cache, enable_cache=enable_cache)
    dict2_megalodon = combine_both_strand_by_meteore(dict_megalodon)

    dict_deepsignal = import_call(infn_deepsignal, "DeepSignal", baseFormat=baseFormat, include_score=False,
                                  siteLevel=False,
                                  using_cache=using_cache, enable_cache=enable_cache)
    dict2_deepsignal = combine_both_strand_by_meteore(dict_deepsignal)

    dict_tombo = import_call(infn_tombo, "Tombo", baseFormat=baseFormat, include_score=False,
                             siteLevel=False,
                             using_cache=using_cache, enable_cache=enable_cache)
    dict2_tombo = combine_both_strand_by_meteore(dict_tombo)

    dict2_list = [dict2_nanopolish, dict2_deepmod, dict2_meteore, dict2_megalodon, dict2_deepsignal, dict2_tombo]
    name_list = ['nanoplish', 'deepmod', 'meteore_rf', 'megalodon', 'deepsignal', 'tombo']

    logger.info("\n\nReport chr1 cpgs:")
    cpgs = [159199943, 159200094, 159200216, 159200386]
    for dict2, name in zip(dict2_list, name_list):
        for cpg in cpgs:
            key = ('chr1', cpg)
            if key in dict2:
                logger.info(f"key={key} in name={name}")
        pass

    dataset = defaultdict(list)
    for key in selected_keys:
        dataset['chr'].append(key[0])
        dataset['pos'].append(key[1])
        for k in range(len(name_list)):
            # if name_list[k] == 'meteore_rf' and (key not in dict2_deepsignal or key not in dict2_megalodon):
            #     dataset[f'{name_list[k]}_freq'].append(None)
            #     dataset[f'{name_list[k]}_cov'].append(None)
            #     continue
            if key in dict2_list[k]:
                dataset[f'{name_list[k]}_freq'].append(dict2_list[k][key][0])
                dataset[f'{name_list[k]}_cov'].append(dict2_list[k][key][1])
            else:
                dataset[f'{name_list[k]}_freq'].append(None)
                dataset[f'{name_list[k]}_cov'].append(None)

    df = pd.DataFrame.from_dict(dataset)
    outfn = os.path.join(pic_base_dir, 'sanity_meteore_deepmod.xlsx')
    df.to_excel(outfn)
    logger.info(f"save to {outfn}")

    pass



#!/usr/bin/env python3

"""
=================================================================
Global variable initializations
=================================================================

Define global directory and loggers
"""

import datetime
import getpass
import logging
import os
import socket

import matplotlib.pyplot as plt

## Modify to the project base dir
project_base_dir = "/projects/li-lab/yang/workspace/nanome"  # project base

data_base_dir = os.path.join(project_base_dir, 'data')  # all used data base
src_base_dir = os.path.join(project_base_dir, 'src')  # source code base

init_log_level_prj = logging.INFO

hostname = socket.gethostname()

if hostname.startswith('winter') or hostname.startswith('sumner'):  # in JAX HPC
    username = getpass.getuser()
    if username == 'liuya':  # output to ly's dirs
        results_dir = "/projects/li-lab/yang/results"  # temp output base
        cache_dir = '/fastscratch/liuya/nanome/cache_dir'  # cache readed object to pkl
        temp_dir = '/fastscratch/liuya/nanome/temp_dir'
    else:  # output to shared dirs
        results_dir = f"/fastscratch/{username}/nanome/nanome_results"  # temp output base
        cache_dir = f'/fastscratch/{username}/nanome/nanome_cache_dir'  # cache readed object to pkl
        temp_dir = f'/fastscratch/{username}/nanome/nanome_temp_dir'
else:
    ## Default output dir set to pwd
    results_dir = os.path.join(os.getcwd(), 'nanome_results')
    cache_dir = os.path.join(os.getcwd(), 'nanome_cache_dir')
    temp_dir = os.path.join(os.getcwd(), 'nanome_temp_dir')

today_str = datetime.date.today().strftime("%Y-%m-%d")

# log_base_dir = os.path.join(results_dir, "log")
pic_base_dir = os.path.join(results_dir, today_str)


# Ensure dir is created if it is not exist
def ensure_dir(dir_name):
    if not os.path.exists(dir_name):
        os.umask(0)
        os.makedirs(dir_name, exist_ok=True)
        # logger.debug("create dir [{}]".format(dir_name))


# create 3 folders if needed
ensure_dir(pic_base_dir)
# ensure_dir(log_base_dir)

# Global loggers can be used any where
logger = logging.getLogger()

formatter_log_results = logging.Formatter(
    '%(asctime)s - [%(filename)s:%(lineno)d] - %(levelname)s: %(message)s')  # datefmt='%Y-%m-%d %H:%M:%S'


def add_logging_file(fn):
    """
    Add file to output for logger
    :param fn:
    :return:
    """
    file_handler = logging.FileHandler(fn)
    file_handler.setFormatter(formatter_log_results)
    logging.getLogger().addHandler(file_handler)
    file_handler.setLevel(logging.DEBUG)


# Must be after log_base_dir var defined
def init_logging():
    """
    Init both stdout and log file located at LOG_DIR/log-YYYY-MM-DD.txt
    Sample: /Users/liuya/results/log/log-11-30.txt
    :return:
    """
    # StreamHandler to screen
    console_handler = logging.StreamHandler()
    console_handler.setLevel(init_log_level_prj)
    console_handler.setFormatter(formatter_log_results)

    # FileHandler to file
    file_handler = None
    # file_handler = logging.FileHandler(os.path.join(log_base_dir, 'log-{}.txt'.format(today_str)))
    # file_handler.setLevel(init_log_level_prj)
    # file_handler.setFormatter(formatter_log_results)

    # add two Handlers
    logger.addHandler(console_handler)
    # logger.addHandler(file_handler)

    # suppress some libs' logging results
    logging.getLogger('matplotlib').setLevel(logging.WARNING)
    logging.getLogger('numexpr').setLevel(logging.WARNING)
    plt.rcParams.update({'figure.max_open_warning': 0})
    return file_handler, console_handler


file_handler, console_handler = init_logging()


def set_log_debug_level():
    """
    Set debug logger level to file and std out
    :return:
    """
    if file_handler is not None:
        file_handler.setLevel(logging.DEBUG)
    console_handler.setLevel(logging.DEBUG)
    logging.getLogger().setLevel(logging.DEBUG)


def set_log_info_level():
    """
    Set info logger level to file and std out
    :return:
    """
    if file_handler is not None:
        file_handler.setLevel(logging.INFO)
    console_handler.setLevel(logging.INFO)
    logging.getLogger().setLevel(logging.INFO)


def set_log_error_level():
    if file_handler is not None:
        file_handler.setLevel(logging.ERROR)
    console_handler.setLevel(logging.ERROR)


def current_time_str():
    # time_str = datetime.datetime.now().strftime("%Y-%m-%d-%H-%M-%S-%f")
    time_str = datetime.datetime.now().strftime("%Y_%m_%d_%H_%M_%S_%f")
    return time_str



#!/usr/bin/env python3
# @Author   : Yang Liu
# @FileName : tss_eval.py
# @Software : NANOME project
# @Organization : JAX Li Lab
# @Website  : https://github.com/TheJacksonLaboratory/nanome

"""
Export read/site-level methylation results for TSS/METEORE analysis in nanome paper.
"""
import argparse
from multiprocessing import Manager

from nanocompare.eval_common import *
from nanocompare.global_settings import get_tool_name, save_done_file, nanome_version


def import_and_save_meteore(callfn, callencode, outfn):
    import_call(callfn, callencode, baseFormat=1, enable_cache=False, using_cache=False,
                include_score=False, siteLevel=False, save_unified_format=True, outfn=outfn,
                filterChr=args.chrs)


def output_calldict_to_unified_bed_as_0base(dictCalls, outfn, sep='\t'):
    """
    Assume dictCalls are key->value, key=(chr, 123, +), value=[(freq, cov), ...], note is 1-based format
    Output is format: 0-based start, 1-base end format with tab-sep for TSS analysis

    ==============================
    chr start end . . freq  cov

    :param dictCalls:
    :param outfn:
    :return:
    """
    ## Change to gzip output
    # with open(outfn, 'w') as outf:
    with gzip.open(outfn, 'wt') as outf:
        for key in dictCalls:
            strlist = [key[0], str(key[1] - 1), str(key[1]), '.', '.', key[2], str(dictCalls[key][0]),
                       str(dictCalls[key][1])]
            outf.write(sep.join(strlist) + '\n')
    logger.debug(f'Output for TSS analysis: {outfn}')


def import_and_save_site_level(callfn, callname, callencode, minToolCovCutt, outfn, ns):
    ontCall = import_call(callfn, callencode, baseFormat=baseFormat, enable_cache=enable_cache,
                          using_cache=using_cache, include_score=False, siteLevel=True, filterChr=args.chrs)

    ontCallWithCov = readLevelToSiteLevelWithCov(ontCall, minCov=minToolCovCutt, toolname=callname)
    ns.callsites[callname] = len(ontCallWithCov)
    output_calldict_to_unified_bed_as_0base(ontCallWithCov, outfn)


def parse_arguments():
    """
    :return:
    """
    parser = argparse.ArgumentParser(prog='tss_eval (NANOME)',
        description='Export read/site level methylation results of all nanopore tools in nanome paper')
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s v{nanome_version}')
    parser.add_argument('--dsname', type=str, help="dataset name", required=True)
    parser.add_argument('--runid', type=str, help="running prefix", required=True)
    parser.add_argument('--calls', nargs='+', help='all ONT call results <tool-name>:<file-name> seperated by spaces',
                        required=True)
    parser.add_argument('--bgtruth', type=str, help="background truth file <encode-type>:<file-name1>;<file-name2>",
                        default=None)
    parser.add_argument('--sep', type=str, help="seperator for output csv file", default='\t')
    parser.add_argument('--processors', type=int, help="running processors", default=1)
    parser.add_argument('-o', type=str, help="output dir", default=pic_base_dir)
    parser.add_argument('--enable-cache', help="if enable cache functions", action='store_true')
    parser.add_argument('--using-cache', help="if use cache files", action='store_true')
    parser.add_argument('--output-unified-format', help="True:output read level results(1-based start), False: output site-level results(0-based start)", action='store_true')
    parser.add_argument('--chrs', nargs='+', help='chromosome list',
                        default=humanChrSet)
    parser.add_argument('--tagname', type=str, help="output unified file tagname", default=None)
    parser.add_argument('--debug', help="if output debug info", action='store_true')
    return parser.parse_args()


if __name__ == '__main__':
    args = parse_arguments()
    if args.debug:
        set_log_debug_level()
    else:
        set_log_info_level()

    logger.debug(f"args={args}")

    if args.output_unified_format:  ## if output METEORE format, must read directly
        enable_cache = False
        using_cache = False
    else:
        enable_cache = args.enable_cache
        using_cache = args.using_cache

    # runid is always like 'MethCorr-K562_WGBS_2Reps', remove first word as RunPrefix like K562_WGBS_2Reps
    RunPrefix = args.runid.replace('TSS-', '')

    minToolCovCutt = 1
    bgtruthCutt = 1

    # Currently we only use 1-base start format, for BED of singletons, non-singletons are use 1-base format
    baseFormat = 1

    # output csv seperator: , or tab
    sep = args.sep

    out_dir = os.path.join(args.o, args.runid)
    os.makedirs(out_dir, exist_ok=True)
    logger.info(f'Output to dir:{out_dir}')

    logger.debug(args)
    logger.debug(f'\n\n####################\n\n')

    if args.output_unified_format:
        ## Output read-level unified format with 1-based start
        logger.info(f"We are outputing each tool's unified results for read-level, same as METEORE format")
        input_list = []
        for callstr in args.calls:
            callencode, callfn = callstr.split(':')
            if len(callfn) == 0:
                continue
            callname = get_tool_name(callencode)
            # Consider tools have read-level outputs, except for DeepMod
            if callname not in ['Nanopolish', 'Megalodon', 'DeepSignal', 'Guppy', 'Tombo']:
                continue
            outfn = os.path.join(out_dir,
                                 f"{args.dsname}_{callname}{f'-{args.tagname}' if args.tagname else ''}-perRead-score.tsv.gz")

            input_list.append((callfn, callencode, outfn,))
        with Pool(processes=args.processors) as pool:
            pool.starmap(import_and_save_meteore, input_list)

        save_done_file(out_dir)
        logger.info("### Unified format output DONE")
        sys.exit(0)

    ## Convert into 0-based format site level bed CpG files, used for TSS plot
    if args.bgtruth:
        logger.debug("We are generating bed CpG results for BG-Truth")
        # we import multiple (1 or 2) replicates and join them
        encode, fnlist = args.bgtruth.split(':')
        fnlist = fnlist.split(';')

        if len(fnlist) > 2:
            raise Exception(f'Currently only support bgtruth with upto two, but found more: {fnlist}')

        logger.debug(f'BGTruth fnlist={fnlist}, encode={encode}')

        bgTruthList = []
        for fn in fnlist:
            if len(fn) == 0:  # incase of input like 'bismark:/a/b/c;'
                continue
            # import if cov >= 1 firstly, then after join two replicates step, remove low coverage
            bgTruth1 = import_bgtruth(fn, encode, covCutoff=1, baseFormat=baseFormat, includeCov=True,
                                      using_cache=using_cache, enable_cache=enable_cache)
            bgTruthList.append(bgTruth1)

        # Combine one/two replicates, using cutoff=1 or 5
        bgTruth = combineBGTruthList(bgTruthList, covCutoff=bgtruthCutt)
        outfn = os.path.join(out_dir, f'{args.dsname}_BSseq-perSite-cov{bgtruthCutt}.bed.gz')
        logger.debug(f'Combined BS-seq data (cov>={bgtruthCutt}), all methylation level sites={len(bgTruth):,}')
        output_calldict_to_unified_bed_as_0base(bgTruth, outfn)
        logger.debug(f'\n\n####################\n\n')

    logger.debug("We are outputing bed CpG results for each tool")

    mgr = Manager()
    ns = mgr.Namespace()

    # callname -> # of sites
    ns.callsites = mgr.dict()

    input_list = []
    for callstr in args.calls:
        # logger.info(f'\n\n####################\n\n')
        callencode, callfn = callstr.split(':')
        if len(callfn) == 0:
            continue
        callname = get_tool_name(callencode)

        outfn = os.path.join(out_dir, f'{args.dsname}_{callname}-perSite-cov{minToolCovCutt}.bed.gz')
        input1 = (callfn, callname, callencode, minToolCovCutt, outfn, ns,)
        input_list.append(input1)

    with Pool(processes=args.processors) as pool:
        pool.starmap(import_and_save_site_level, input_list)

    for key in ns.callsites.keys():
        logger.debug(f"tool={key}, sites={ns.callsites[key]}")
    save_done_file(out_dir)
    logger.info("TSS bed file generation DONE")



#!/usr/bin/env python3
# @Author   : Yang Liu
# @FileName : gen_html_report.py
# @Software : NANOME project
# @Organization : JAX Li Lab
# @Website  : https://github.com/TheJacksonLaboratory/nanome

import glob
import os
import sys
from collections import defaultdict

import matplotlib.pyplot as plt
import pandas as pd
from jinja2 import Environment, FileSystemLoader

from nanocompare.global_settings import ToolNameList


def density_plot(df, outfn, tool, col_index=6, bins=None, the_range=None, x_label=None):
    # setting font size
    plt.rcParams.update({'font.size': 6})

    # plots
    fig, ax = plt.subplots(figsize=(2, 2))
    ax.hist(df.iloc[:, col_index], bins=bins, range=the_range, color='blue', alpha=0.4)

    # Add labels
    if the_range:
        plt.xlim([0, 1])
    plt.title(f'{tool} = {len(df):,} CpGs')
    plt.xlabel(x_label)
    plt.ylabel('# CpGs')

    # save figures
    os.makedirs(os.path.dirname(outfn), exist_ok=True)
    plt.savefig(outfn, dpi=300, bbox_inches='tight')
    plt.close()
    print(f"save to {outfn}")


def get_methcall_report_df(baseDir, outDir):
    """
    Generate the methylation calling report dataframe
    :param baseDir:
    :return:
    """
    ret_dict = defaultdict(list)
    for tool in ToolNameList:
        fnlist = glob.glob(os.path.join(baseDir, f'*_{tool}-perSite-cov1.sort.bed.gz'))
        if len(fnlist) < 1:
            print(f"Not found file in baseDir={baseDir}, pattern={f'*_{tool}-perSite-cov1.sort.bed.gz'}")
            ret_dict['Tool'].append(tool)
            ret_dict['CpGs'].append(f"0")
            continue
        try:
            df_site_level = pd.read_csv(fnlist[0], sep='\t', header=None, index_col=False)
            ret_dict['Tool'].append(tool)
            ret_dict['CpGs'].append(f"{len(df_site_level):,}")

            outfn = os.path.join(outDir, 'images', f'dist_{tool}.png')
            density_plot(df_site_level, outfn, tool, bins=10, the_range=(0,1), x_label='Methylation %')

            outfn = os.path.join(outDir, 'images', f'cov_{tool}.png')
            density_plot(df_site_level, outfn, tool, col_index=7, x_label='Coverage')
        except:  # can not call any results
            ret_dict['Tool'].append(tool)
            ret_dict['CpGs'].append(f"0")
    return pd.DataFrame.from_dict(ret_dict)


if __name__ == '__main__':
    dsname = sys.argv[1]
    infn_running_info = sys.argv[2]
    infn_basecall_info = sys.argv[3]
    indir_methcall = sys.argv[4]
    outdir = sys.argv[5]

    ## Running information summary
    df_running_info = pd.read_csv(infn_running_info, sep='\t', )
    # print(df_running_info.to_html())

    ## Basecalling report summary
    dataset_basecall = defaultdict(list)
    with open(infn_basecall_info) as stats:
        for line in stats:
            linesplit = line.strip().split(':')
            linesplit = [e.strip() for e in linesplit]
            if line.startswith('Top 5'):
                break
            if len(linesplit) > 1:
                dataset_basecall['Title'].append(linesplit[0])
                dataset_basecall['Information'].append(linesplit[1])
            else:
                dataset_basecall['Title'].append(linesplit[0])
                dataset_basecall['Information'].append("")

    df_basecall_info = pd.DataFrame.from_dict(dataset_basecall)
    print(df_basecall_info)

    df_methcall_info = get_methcall_report_df(indir_methcall, outdir)
    print(df_methcall_info)

    env = Environment(loader=FileSystemLoader('src/nanocompare/report'))
    template = env.get_template('index.html')

    filename = os.path.join(outdir, 'nanome_report.html')
    with open(filename, 'w') as fh:
        fh.write(template.render(
            dsname=dsname,
            df_running_info=df_running_info,
            df_basecall_info=df_basecall_info,
            df_methcall_info=df_methcall_info,
        ))

    # df_basecall_info = pd.read_csv(infn_basecall_info, sep='\t')
    # print(df_basecall_info)

    pass



#!/usr/bin/env python3
# @Author   : Yang Liu
# @FileName : FilesSeparator.py
# @Software : NANOME project
# @Organization : JAX Li Lab
# @Website  : https://github.com/TheJacksonLaboratory/nanome

"""
Seperate all fast5 file into more subfolders 1--N,
or a flexible number of subfolders that each with N files

Usage:
    sbatch FilesSeparator.sh \
        -i /fastscratch/liuya/nanocompare/hl60_input/HL60-Nanopore_GT18-07373.fast5.tar \
        -o /fastscratch/liuya/nanocompare/hl60_output_test \
        -n 10 --tar-file

    bash FilesSeparator.sh \
       -i ../test_data/demo1.fast5.reads.tar.gz  \
       -o /fastscratch/liuya/nanocompare/demo1_output \
       -n 50  --tar-file --fix-folder-size

"""

import argparse
import os
import shutil
import subprocess
from glob import glob
from multiprocessing import Pool

import math
from tqdm import tqdm


def recursive_find_fast5(basedir):
    """
    Find all fast5 files recursively in folder
    """
    fast5files = []
    for root, dirs, files in os.walk(basedir):
        for file in files:
            if file.endswith(".fast5"):
                fast5files.append(os.path.join(root, file))
    return fast5files


def main():
    target_num = args.n
    outdir = os.path.join(args.o, 'sept_dir')
    os.makedirs(outdir, exist_ok=True)
    prefix_str = args.prefix_name
    fast5files = recursive_find_fast5(indir)  # [1,2,3,4,5,6,7,8,9,0]
    print(f'total files: {len(fast5files)}, indir={indir}, TargetNum={target_num}, outdir={outdir}', flush=True)

    if args.fix_folder_size:
        num_folders = math.ceil(len(fast5files) / args.n)
        print(f"Fixed folder size={args.n}, total folders={num_folders}", flush=True)

        for k in range(num_folders):
            destdir = os.path.join(outdir, f'{prefix_str}{(k % target_num) + 1}')
            os.makedirs(destdir, exist_ok=True)
            for fn in fast5files[k * args.n: (k + 1) * args.n]:
                if args.copy:
                    shutil.copy(fn, destdir)
                else:
                    shutil.move(fn, destdir)
    else:
        print(f"Fixed number of folders={args.n}", flush=True)
        for k in range(target_num):
            fdir = os.path.join(outdir, f'{prefix_str}{k + 1}')
            if not os.path.exists(fdir):
                os.umask(0)
                os.makedirs(fdir, exist_ok=True)

        print(f"Start seperate into N={target_num}", flush=True)
        for k, fn in tqdm(enumerate(fast5files)):
            destdir = os.path.join(outdir, f'{prefix_str}{(k % target_num) + 1}')
            if args.copy:
                shutil.copy(fn, destdir)
            else:
                shutil.move(fn, destdir)


def untar_file(fn, outdir):
    """
    Untar fn into outdir
    Args:
        fn:
        outdir:

    Returns:

    """
    if fn.endswith('.tar'):
        commandStr = f"tar -xf {fn} -C {outdir}"
    elif fn.endswith('.tar.gz'):
        commandStr = f"tar -xzf {fn} -C {outdir}"
    else:
        raise Exception(f"{fn} type is not support")
    process = subprocess.Popen(commandStr,
                               stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                               shell=True)
    process.communicate()
    print(f"### Untar finished for {fn}", flush=True)


def parse_arguments():
    parser = argparse.ArgumentParser(prog='FileSeperator (NANOME)', description='File seperator for fast5/tar/tar.gz')
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s v1.0')
    parser.add_argument('-i', type=str, help="input dir/file", required=True)
    parser.add_argument('-o', type=str, help="output dir", required=True)
    parser.add_argument('-n', type=int, help="number of seperated folders", required=True)
    parser.add_argument('-p', type=int, help="number of processors", default=8)
    parser.add_argument('--prefix-name', type=str, help="subfolder prefix string", default="")
    parser.add_argument('--copy', action='store_true', help="true if using copy, else using move")
    parser.add_argument('--tar-file', action='store_true',
                        help="true if deal with tar/tar.gz files, else are fast5 files")
    parser.add_argument('--fix-folder-size', action='store_true',
                        help="true if fix each subfolder contain at most N files, else fix number of folders as N")
    return parser.parse_args()


if __name__ == "__main__":
    args = parse_arguments()
    print(f"args={args}")

    if args.tar_file:  # tar files
        flist = []
        if os.path.isdir(args.i):
            flist += glob(os.path.join(args.i, '**', '*.tar'), recursive=True)
            flist += glob(os.path.join(args.i, '**', '*.tar.gz'), recursive=True)
        elif os.path.isfile(args.i):
            flist += [args.i]
        print(f"### Find tar file list={flist}", flush=True)

        untar_dir = os.path.join(args.o, 'untar_dir')
        os.makedirs(untar_dir, exist_ok=True)

        arg_list = []
        for fn in flist:
            # untar_file(fn, untar_dir)
            arg_list.append((fn, untar_dir,))
        with Pool(args.p) as p:
            p.starmap(untar_file, arg_list)
        indir = untar_dir
    else:  # fast5 files
        indir = args.i

    main()
    print("### FilesSeparator DONE")



#!/usr/bin/env python3

"""
Construct the benchmarking data for resource usage evaluation
"""
import glob
import os
import random
from shutil import copy

import numpy as np
import pandas as pd
from pandas.api.types import CategoricalDtype

from nanocompare.global_config import set_log_debug_level, logger, pic_base_dir

base_input = '/projects/li-lab/Nanopore_compare/nanopore_fast5/NA19240-N300-sept/10'
out_dir = os.path.join(pic_base_dir, 'BenchmarkingData')

# trace_fn = '/projects/li-lab/Nanopore_compare/suppdata/benchmarking/benchmarking-trace.txt'
trace_fn = '/projects/li-lab/Nanopore_compare/result/benchmarking/nanome.pipeline_trace.txt'
# sizes = [int(x) for x in np.linspace(1000, 10000, 10)]

sample_sizes = [int(x) for x in np.linspace(800, 8000, 10)]

random.seed(688)

tools_cat_dtype = CategoricalDtype(categories=["DeepSignal", "Tombo", "Nanopolish", "DeepMod", "Megalodon", "Basecall", "Resquiggle"], ordered=True)


def gen_benchmarking_data():
    logger.info(f'gen_benchmarking_data for sample size={sample_sizes}')
    fnlist = glob.glob(os.path.join(base_input, '*.fast5'))

    logger.info(len(fnlist))
    os.makedirs(out_dir, exist_ok=True)

    for t in sample_sizes:
        retfnlist = random.sample(fnlist, k=t)

        benchDir = os.path.join(out_dir, f'MB{t:02n}K')
        os.makedirs(benchDir, exist_ok=True)

        for fn in retfnlist:
            # copy fn into benchDir
            copy(fn, benchDir)
        logger.info(f'Copy done for t={t}, benchDir={benchDir}')


def parse_mem_str_to_gbsize(strmem):
    """
    String like 845 MB	677.8 MB to GB size
    :param strtime:
    :return:
    """
    strmem = strmem.strip()

    if strmem.endswith('MB'):
        memgb = float(strmem[:-2]) / 1024
    elif strmem.endswith('GB'):
        memgb = float(strmem[:-2])
    elif strmem.endswith('KB'):
        memgb = float(strmem[:-2]) / 1024 / 1024
    else:
        raise Exception(f'Not recognized strmem={strmem}')
    return memgb

    pass


def parse_string_time_to_seconds(strtime):
    """
    String like 1h 52m 55s to seconds
    :param strtime:
    :return:
    """
    strlist = strtime.strip().split(' ')
    secs = float(0)
    for strk in strlist:

        if strk.endswith('h'):
            num = int(float(strk[:-1]))
            secs += 60 * 60 * num
        elif strk.endswith('m'):
            num = int(float(strk[:-1]))
            secs += 60 * num
        elif strk.endswith('ms'):
            num = int(float(strk[:-2]))
            secs += num / 1000
        elif strk.endswith('s'):
            num = int(float(strk[:-1]))
            secs += num
        else:
            raise Exception(f"not recognized format strtime={strtime}")
    return secs


def extract_tool_and_dsname_from_name(row):
    """
    Extract Basecall (MB1.6K) into Basecall and MB1.6K, and 1600 three fields
    :param instr:
    :return:
    """
    try:
        toolname, dsname = row['name'].strip().split(' ')
        dsname = dsname[1:-1]
    except:  # No tag process
        toolname = row['name'].strip()
        dsname = 'None'
    if not dsname.startswith('MB'):  # not tagged with MB
        dsname = 'None'
        reads = 0
    else:
        reads = float(dsname[2:-1])

    duration = parse_string_time_to_seconds(row['duration'])
    realtime = parse_string_time_to_seconds(row['realtime'])

    cpu = float(row['%cpu'][:-1])
    peak_rss = parse_mem_str_to_gbsize(row['peak_rss'])
    peak_vmem = parse_mem_str_to_gbsize(row['peak_vmem'])
    rchar = parse_mem_str_to_gbsize(row['rchar'])
    wchar = parse_mem_str_to_gbsize(row['wchar'])

    return toolname, dsname, reads, duration, realtime, cpu, peak_rss, peak_vmem, rchar, wchar


def analyse_trace():
    df = pd.read_csv(trace_fn, sep='\t')
    logger.info(df)

    ## Extract all meaning fields into raw format
    # return toolname, dsname, reads, duration, realtime, cpu, peak_rss, peak_vmem, rchar, wchar
    df[['tool', 'dsname', 'reads', 'duration', 'realtime', '%cpu', 'peak_rss', 'peak_vmem', 'rchar', 'wchar']] = df.apply(extract_tool_and_dsname_from_name, axis=1, result_type="expand")

    df['reads'] = df['reads'].astype(np.int)

    df['tool'] = df['tool'].astype(tools_cat_dtype)
    df = df.sort_values(by=['tool', 'reads'])

    outfn = os.path.join(pic_base_dir, 'benchmarking.log.formated.table.step1.all.steps.csv')
    df.to_csv(outfn, sep=',')

    ## Recalculate the time and memory for each tools
    for index, row in df.iterrows():
        if row['tool'] in ['DeepSignal', 'Tombo', 'DeepMod', 'Nanopolish']:
            # Get basecalled row
            basecallRow = df.loc[(df.toolName == 'Basecall') & (df.dsname == row['dsname']), :].iloc[0, :]
            df.at[index, 'duration'] = df.at[index, 'duration'] + basecallRow['duration']
            df.at[index, 'realtime'] = df.at[index, 'realtime'] + basecallRow['realtime']

            df.at[index, 'peak_rss'] = max(df.at[index, 'peak_rss'], basecallRow['peak_rss'])
            df.at[index, 'peak_vmem'] = max(df.at[index, 'peak_vmem'], basecallRow['peak_vmem'])
            df.at[index, 'rchar'] = max(df.at[index, 'rchar'], basecallRow['rchar'])
            df.at[index, 'wchar'] = max(df.at[index, 'wchar'], basecallRow['wchar'])
        if row['tool'] in ['DeepSignal', 'Tombo']:
            # Get resquiggle row
            resquiggleRow = df.loc[(df.toolName == 'Resquiggle') & (df.dsname == row['dsname']), :].iloc[0, :]
            df.at[index, 'duration'] = df.at[index, 'duration'] + resquiggleRow['duration']
            df.at[index, 'realtime'] = df.at[index, 'realtime'] + resquiggleRow['realtime']

            df.at[index, 'peak_rss'] = max(df.at[index, 'peak_rss'], resquiggleRow['peak_rss'])  # TODO why Resquiggle costs too much memory, need / 6
            df.at[index, 'peak_vmem'] = max(df.at[index, 'peak_vmem'], resquiggleRow['peak_vmem'])  # TODO why Resquiggle costs too much memory, need / 6
            df.at[index, 'rchar'] = max(df.at[index, 'rchar'], resquiggleRow['rchar'])
            df.at[index, 'wchar'] = max(df.at[index, 'wchar'], resquiggleRow['wchar'])

    ## Filter out non-tool rows
    df = df[df.toolName.isin(['DeepSignal', 'Tombo', 'DeepMod', 'Nanopolish', 'Megalodon'])]

    df = df.sort_values(by=['tool', 'reads'])
    outfn = os.path.join(pic_base_dir, 'benchmarking.log.formated.table.step2.all.tools.csv')
    df.to_csv(outfn, sep=',')
    pass


if __name__ == '__main__':
    set_log_debug_level()
    # gen_benchmarking_data()
    analyse_trace()



import urllib  # the lib that handles the url stuff
import urllib.request
import re

in_html = "https://raw.githubusercontent.com/nanopore-wgs-consortium/NA12878/master/nanopore-human-genome/rel_3_4.md"

if __name__ == '__main__':
    with urllib.request.urlopen(in_html) as response:
        html = response.read()
    html_str = str(html)

    ## Cope with: http://s3.amazonaws.com/nanopore-human-wgs/rel3-fast5-chr1.part01.tar
    flist = re.findall(r"http://s3\.amazonaws\.com/nanopore-human-wgs/rel\d-fast5-chr[0-9XYM]+\.part\d+\.tar", html_str)

    print(len(flist))

    with open("na12878.filelist.txt", 'w') as outf:
        for fn in flist:
            outf.write(f'{fn}\n')











#!/usr/bin/env python3

"""
Define names and global variables
"""

import os

from nanocompare.global_config import data_base_dir

nanome_version="1.3.2"

# define the small error of 0 and 1, for fully-meth and unmeth eval
epslong = 1e-5

# can be 1.0, or 0.9 for the level of fully-meth definition
fully_meth_level = 1.0

# chr 1-22 X and Y
humanChrSet = [f'chr{k}' for k in range(1, 23)] + ['chrX', 'chrY']

ecoliChrSet = ['NC_000913.3']

datasets_order = ["NA12878", "NA19240", "APL", "K562", "HL60"]

ToolNameList = ['Nanopolish', 'Megalodon', 'DeepSignal', 'Guppy', 'Tombo', 'METEORE', 'DeepMod']

Top3ToolNameList = ToolNameList[:3]

ToolEncodeList = ['DeepSignal', 'Tombo', 'Nanopolish', 'DeepMod.C', 'DeepMod.Cluster',
                  'Megalodon', 'Megalodon.ZW',
                  'Guppy', 'Guppy.ZW', 'Guppy.gcf52ref', 'METEORE']

BGTruthEncodeList = ['bismark', 'encode']

# ToolsColorList = ["#999999", "#E69F00", "#56B4E9", "#009E73", "#CC79A7", "#0072B2", "#D55E00", "#F0E442"]
ToolsColorList = ["#56B4E9", "#CC79A7", "#999999", "#009E73", "#E69F00", "#0072B2", "#D55E00"]

referenceGenomeFile = "/projects/li-lab/Nanopore_compare/nf_input/reference_genome/hg38/hg38.fasta"

singletonsFile = "hg38_singletons_10bp.bed.gz"
nonsingletonsFile = "hg38_nonsingletons_10bp.bed.gz"
singletonFileExtStr = "_10bp.bed.gz"

narrowCoordNameList = ['x.x.Genome-wide',
                       singletonsFile, nonsingletonsFile,
                       "ONT.hg38.promoterFeature.flank_2000.bed.gz", "ONT.hg38.exonFeature.bed.gz",
                       "ONT.hg38.intronFeature.bed.gz", "ONT.hg38.intergenic.bed.gz",
                       "ONT.hg38.cpgIslandExt.bed.gz", "ONT.hg38.cpgShoresExt.bed.gz",
                       "ONT.hg38.cpgShelvesExt.bed.gz"]

# None means no coordinate used, i.e. Genome-wide
narrowCoordFileList = [None] + [os.path.join(data_base_dir, 'genome-annotation', cofn) for cofn in
                                narrowCoordNameList[1:]]

# CG density bed regions file basename
cg_density_coord_name_list = ["hg38.gc5Base_merged.cg_bin20.bed.gz", "hg38.gc5Base_merged.cg_bin40.bed.gz",
                              "hg38.gc5Base_merged.cg_bin60.bed.gz", "hg38.gc5Base_merged.cg_bin80.bed.gz",
                              "hg38.gc5Base_merged.cg_bin100.bed.gz"]
# CG density bed regions file full path
cg_density_file_list = [os.path.join(data_base_dir, 'genome-annotation', cofn) for cofn in cg_density_coord_name_list]

# Repetitive regions
rep_coord_name_list = ["hg38.repetitive.rep_SINE.bed.gz", "hg38.repetitive.rep_LINE.bed.gz",
                       "hg38.repetitive.rep_LTR.bed.gz", "hg38.repetitive.rep_DNA.bed.gz",
                       "hg38.repetitive.rep_Others.bed.gz"]
rep_file_list = [os.path.join(data_base_dir, 'genome-annotation', cofn) for cofn in rep_coord_name_list]

region_name_to_fn_dict = {
    'Genome-wide': narrowCoordFileList[0], 'Singletons': narrowCoordFileList[1],
    'Non-singletons': narrowCoordFileList[2],
    'Promoters': narrowCoordFileList[3], 'Exons': narrowCoordFileList[4], 'Introns': narrowCoordFileList[5],
    'Intergenic': narrowCoordFileList[6], 'CpG islands': narrowCoordFileList[7], 'CpG shores': narrowCoordFileList[8],
    'CpG shelves': narrowCoordFileList[9],
    'CG_20': cg_density_file_list[0], 'CG_40': cg_density_file_list[1],
    'CG_60': cg_density_file_list[2], 'CG_80': cg_density_file_list[3],
    'CG_100': cg_density_file_list[4],
    'rep_SINE': rep_file_list[0], 'rep_LINE': rep_file_list[1],
    'rep_LTR': rep_file_list[2], 'rep_DNA': rep_file_list[3],
    'rep_Others': rep_file_list[4]
}

# List of all start is 0-based bed files
list_base0_bed_basefn = ["ONT.hg38.cpgIslandExt.bed.gz", "ONT.hg38.cpgShoresExt.bed.gz",
                         "ONT.hg38.cpgShelvesExt.bed.gz"] + \
                        ["hg38.gc5Base_merged.cg_bin20.bed.gz", "hg38.gc5Base_merged.cg_bin40.bed.gz",
                         "hg38.gc5Base_merged.cg_bin60.bed.gz", "hg38.gc5Base_merged.cg_bin80.bed.gz",
                         "hg38.gc5Base_merged.cg_bin100.bed.gz"] + \
                        ["hg38.repetitive.rep_SINE.bed.gz", "hg38.repetitive.rep_LINE.bed.gz",
                         "hg38.repetitive.rep_LTR.bed.gz", "hg38.repetitive.rep_DNA.bed.gz",
                         "hg38.repetitive.rep_Others.bed.gz"]
enable_base_detection_bedfile = True
# enable_base_detection_bedfile = False


# Map each bed file name to a standard name, used by curve data plotting
location_filename_to_abbvname = {
    'x.x.Genome-wide': 'Genome-wide',
    singletonsFile: 'Singletons',
    nonsingletonsFile: 'Non-singletons',
    "ONT.hg38.cpgIslandExt.bed.gz": "CpG islands",
    "ONT.hg38.cpgShoresExt.bed.gz": 'CpG shores',
    "ONT.hg38.cpgShelvesExt.bed.gz": 'CpG shelves',
    "ONT.hg38.exonFeature.bed.gz": 'Exons',
    "ONT.hg38.intergenic.bed.gz": 'Intergenic',
    "ONT.hg38.intronFeature.bed.gz": 'Introns',
    "ONT.hg38.promoterFeature.flank_2000.bed.gz": 'Promoters',

    'hg38_nonsingletons.concordant.bed.gz': 'Concordant',
    'hg38_nonsingletons.discordant.bed.gz': 'Discordant',

    'hg38.gc5Base_merged.cg_bin20.bed.gz': 'CG_20',
    'hg38.gc5Base_merged.cg_bin40.bed.gz': 'CG_40',
    'hg38.gc5Base_merged.cg_bin60.bed.gz': 'CG_60',
    'hg38.gc5Base_merged.cg_bin80.bed.gz': 'CG_80',
    'hg38.gc5Base_merged.cg_bin100.bed.gz': 'CG_100',

    'hg38.repetitive.rep_SINE.bed.gz': 'rep_SINE',
    'hg38.repetitive.rep_LINE.bed.gz': 'rep_LINE',
    'hg38.repetitive.rep_LTR.bed.gz': 'rep_LTR',
    'hg38.repetitive.rep_DNA.bed.gz': 'rep_DNA',
    'hg38.repetitive.rep_Others.bed.gz': 'rep_Others',
}

# Tagname list of coordinate file list
narrowCoordFileTag = ['Genome-wide'] + [location_filename_to_abbvname[cofn] for cofn in narrowCoordNameList[1:]]
cgCoordFileTag = [location_filename_to_abbvname[cofn] for cofn in cg_density_coord_name_list]
repCoordFileTag = [location_filename_to_abbvname[cofn] for cofn in rep_coord_name_list]

# which column of performance table is extracted and returned
perf_report_columns = ['Dataset', 'Tool', 'Location', 'Accuracy', "Macro-F1", 'ROC-AUC', 'Average-Precision',
                       "Macro-Precision", "Macro-Recall", "Micro-F1", "Micro-Precision", "Micro-Recall", 'F1_5mC',
                       'F1_5C', 'Precision_5mC', 'Precision_5C', 'Recall_5mC', 'Recall_5C', 'mCsites_called',
                       'Csites_called', 'mCsites', 'Csites', 'referenceCpGs', 'prefix',
                       'coord']

# Rename raw name of region file (third section of .) to print name, such as 'ONT.hg38.cpgIslandExt.bed'
location_name_map_raw_to_standard = {
    'cpgIslandExt': 'CpG islands',
    'discordant': 'Discordant',
    'concordant': 'Concordant',
    'cpgShoresExt': 'CpG shores',
    'cpgShelvesExt': 'CpG shelves',
    'exonFeature': 'Exons',
    'intergenic': 'Intergenic',
    'intronFeature': 'Introns',
    'promoterFeature2000': 'Promoters',  # We use 2k bp promoter bed region
    'geneFeature': 'GeneFeature',
    'cg_bin20': 'CG_20',  # CG density 20%
    'cg_bin40': 'CG_40',
    'cg_bin60': 'CG_60',
    'cg_bin80': 'CG_80',
    'cg_bin100': 'CG_100'
}

locations_category = ["Promoters", "Exons", "Introns",
                      "Intergenic", "CpG islands",
                      "CpG shores", "CpG shelves"]

locations_singleton = ["Genome-wide", "Singletons", "Non-singletons",
                       "Concordant", "Discordant"]

# New introduced regions for CG density and repetitive
locations_new = ["CG_20", "CG_40", "CG_60", "CG_80", "CG_100"] + \
                ["rep_SINE", "rep_LINE", "rep_LTR", "rep_DNA", "rep_Others"]

locations_order = ["Genome-wide", "Singletons", "Non-singletons", "Concordant", "Discordant"] + \
                  ["Promoters", "Exons", "Introns", "Intergenic", "CpG islands", "CpG shores", "CpG shelves"] + \
                  ["CG_20", "CG_40", "CG_60", "CG_80", "CG_100"] + \
                  ["rep_SINE", "rep_LINE", "rep_LTR", "rep_DNA", "rep_Others"]


def get_tool_name(encode_name):
    if encode_name.find('.') != -1:  # cut of DeepMod.C or DeepMod.Cluster
        return encode_name[:encode_name.find('.')]
    return encode_name


def rename_location_from_coordinate_name(df):
    """
    Rename and change raw values of report df to more meaning full for display
    :param df:
    :return:
    """
    df = df.replace(to_replace=singletonsFile, value="x.x.Singletons")
    df = df.replace(to_replace=nonsingletonsFile, value="x.x.Non-singletons")
    df['coord'] = df['coord'].str.replace("promoterFeature.flank_",
                                          "promoterFeature")  # merge third section together for promoter file

    # coord file like: x.x.Singletons, x.x.Non-singletons, HL60_RRBS_2Reps.hg38_nonsingletons.discordant.bed
    # we select the third of . split sections
    df["Location"] = df["coord"].str.split(".", n=3, expand=True)[2]
    df['Location'] = df['Location'].replace(location_name_map_raw_to_standard)
    return df


def save_done_file(outdir, filename="DONE.txt"):
    """
    Save a done file for finish flag
    :param outdir:
    :param filename:
    :return:
    """
    outfn = os.path.join(outdir, filename)
    with open(outfn, "w") as outf:
        outf.write("DONE\n")


if __name__ == '__main__':
    pass



#!/usr/bin/env python3

"""
Sanity check NA12878 for DeepMod.
"""
import argparse
from multiprocessing import Pool

from nanocompare.eval_common import *
from sklearn.metrics import accuracy_score
from sklearn.metrics import f1_score

def report_performance_deepmod(ontCall, bgTruth, threshold=0.5):
    """
    Join with bgtruth, then evaluate
    :param ontCall:
    :param bgTruth:
    :return:
    """
    evalSet = set(ontCall.keys()).intersection(set(bgTruth.keys()))

    y_truth = []
    y_pred = []
    for cpg in evalSet:
        y_truth.append(bgTruth[cpg][0])
        y_pred.append(1 if ontCall[cpg][0] > threshold else 0)
    methCnt = sum(y_truth)
    unmethCnt = len(y_truth) - methCnt
    prec = precision_score(y_truth, y_pred)
    recall = recall_score(y_truth, y_pred)

    f1 = f1_score(y_truth, y_pred)
    accuracy = accuracy_score(y_truth, y_pred)
    ret = {"Precision": prec, "Recall": recall,
            "F1"      : f1, "Accuracy": accuracy, "Threshold": threshold,
            "Unmeth"  : unmethCnt, "Meth": methCnt}

    return ret


def report_meth_unmeth_table_by_chr(chr):
    logger.debug(f'Start for chr={chr}')
    chrSet = [chr]
    # Combine one/two replicates using DeepMod methods
    bgTruth1, ret1 = combineBGTruthList_by_DeepModPaper(bgTruthList, covCutoff=1, filterChrs=chrSet)

    bgTruth5 = {key: bgTruth1[key] for key in bgTruth1 if bgTruth1[key][1] >= 5}
    methCnt5 = sum([bgTruth5[key][0] for key in bgTruth5])
    ret5 = (methCnt5, len(bgTruth5) - methCnt5)

    bgTruth10 = {key: bgTruth1[key] for key in bgTruth1 if bgTruth1[key][1] >= 10}
    methCnt10 = sum([bgTruth10[key][0] for key in bgTruth10])
    ret10 = (methCnt10, len(bgTruth10) - methCnt10)

    ret = (chr, ret1[2], ret1[1], ret1[0], ret5[1], ret5[0], ret10[1], ret10[0])
    logger.debug(f'Finished for chr={chr}, ret={ret}')
    return ret


def report_meth_unmeth_table():
    chrSet = list(humanChrSet)
    chrSet.remove('chrY')
    chrSet.remove('chr22')
    logger.debug(f'chrSet={chrSet}')

    with Pool(args.processors) as pool:
        retList = pool.map(report_meth_unmeth_table_by_chr, chrSet)

    dataset = defaultdict(list)
    for ret in retList:
        dataset['chr'].append(ret[0])
        dataset['Not-used.cov1'].append(ret[1])
        dataset['Unmeth.cov1'].append(ret[2])
        dataset['Meth.cov1'].append(ret[3])

        dataset['Unmeth.cov5'].append(ret[4])
        dataset['Meth.cov5'].append(ret[5])

        dataset['Unmeth.cov10'].append(ret[6])
        dataset['Meth.cov10'].append(ret[7])

    df = pd.DataFrame.from_dict(dataset)
    logger.info(df)
    outfn = os.path.join(out_dir, 'NA12878.all.chr.table.csv')
    df.to_csv(outfn)
    logger.info(f'save to {outfn}')
    pass


def parse_arguments():
    """
    :return:
    """
    parser = argparse.ArgumentParser(description='Sanity check')
    parser.add_argument('--calls', nargs='+', help='all ONT call results <tool-name>:<file-name> seperated by spaces', required=True)
    parser.add_argument('--bgtruth', type=str, help="background truth file <encode-type>:<file-name1>;<file-name1>", required=True)
    parser.add_argument('--dsname', type=str, help="dataset name", required=True)
    parser.add_argument('--runid', type=str, help="running prefix", required=True)
    parser.add_argument('--beddir', type=str, help="base dir for bed files", default=None)  # need perform performance evaluation before, then get concordant, etc. bed files, like '/projects/li-lab/yang/results/2021-04-01'
    parser.add_argument('--sep', type=str, help="seperator for output csv file", default=' ')
    parser.add_argument('--chr', type=str, help="chromosome to evaluate", default=None)
    parser.add_argument('--processors', type=int, help="running processors", default=8)
    parser.add_argument('--pred-threshold', type=float, help="prediction threshold", default=0.5)
    parser.add_argument('--min-bgtruth-cov', type=int, help="cutoff of coverage in bg-truth", default=1)
    parser.add_argument('--toolcov-cutoff', type=int, help="cutoff of coverage in nanopore calls", default=1)
    parser.add_argument('--baseFormat', type=int, help="base format after imported", default=1)
    parser.add_argument('-o', type=str, help="output dir", default=pic_base_dir)
    parser.add_argument('--enable-cache', action='store_true')
    parser.add_argument('--using-cache', action='store_true')
    parser.add_argument('--is-report', action='store_true')
    parser.add_argument('--deepmod-paper-results', action='store_true')

    return parser.parse_args()


if __name__ == '__main__':
    set_log_debug_level()

    args = parse_arguments()

    # cache functions
    enable_cache = args.enable_cache
    using_cache = args.using_cache

    if enable_cache:
        os.makedirs(cache_dir, exist_ok=True)

    RunPrefix = args.runid.replace('SanityCheck-', '')

    ## Now, we are exporting >=1 cov results, the cov = 3,5, etc. can be later used before plotting
    # tool coverage cutoff 1, or 3, 5
    minToolCovCutt = args.toolcov_cutoff
    minToolCovCutt = 1

    # bgtruth coverage cutoff 1, or 5, 10  --min-bgtruth-cov
    bgtruthCutt = args.min_bgtruth_cov
    bgtruthCutt = 1

    # load into program format 0-base or 1-base
    baseFormat = args.baseFormat
    # Currently we only use 1-base start format, for BED of singletons, non-singletons are use 1-base format
    baseFormat = 1

    # output csv seperator: , or tab
    sep = args.sep

    out_dir = os.path.join(args.o, args.runid)
    os.makedirs(out_dir, exist_ok=True)
    logger.info(f'Output to dir:{out_dir}')

    logger.debug(args)
    logger.info(f'\n\n####################\n\n')

    # we import multiple (1 or 2) replicates and join them
    encode, fnlist = args.bgtruth.split(':')
    fnlist = fnlist.split(';')

    if len(fnlist) > 2:
        raise Exception(f'Currently only support bgtruth with upto two, but found more: {fnlist}')

    logger.debug(f'BGTruth fnlist={fnlist}, encode={encode}')

    bgTruthList = []
    for fn in fnlist:
        if len(fn) == 0:  # incase of input like 'bismark:/a/b/c;'
            continue
        # import if cov >= 1 firstly, then after join two replicates step, remove low coverage
        bgTruth_the1 = import_bgtruth(fn, encode, covCutoff=1, baseFormat=baseFormat, includeCov=True, using_cache=using_cache, enable_cache=enable_cache)
        bgTruthList.append(bgTruth_the1)

    logger.info(f'\n\n####################\n\n')
    if args.is_report:
        report_meth_unmeth_table()
        sys.exit(0)

    if args.deepmod_paper_results:
        ## Evaluate for chr
        logger.debug(f'Start check paper results for chr={args.chr}')
        chrSet = [args.chr]
        # Combine one/two replicates using DeepMod methods
        bgTruth1, ret1 = combineBGTruthList_by_DeepModPaper(bgTruthList, covCutoff=1, filterChrs=chrSet)

        # value is (meth_indicator: int, cov)
        bgTruth5 = {key: bgTruth1[key] for key in bgTruth1 if bgTruth1[key][1] >= 5}

        for callstr in args.calls:
            callencode, callfn = callstr.split(':')
            if len(callfn) == 0:
                continue
            if callencode != 'DeepMod.Cluster':
                raise Exception("Only Deepmod paper results")
            # value is (meth_freq, cov)
            ontCall = import_call(callfn, callencode, baseFormat=baseFormat, enable_cache=enable_cache,
                                      using_cache=using_cache, include_score=False, siteLevel=True)
        callSet = set(ontCall.keys())
        bsSet = set(bgTruth5.keys())
        joinedSet = callSet.intersection(bsSet)
        logger.info(f"BS-seq(cov>=5)={len(bgTruth5):,}, DeepMod={len(ontCall):,}, intersect={len(joinedSet):,}")

        ## joined sets performance report
        y_truth = []
        y_pred = []
        for key in joinedSet:
            bs_label=bgTruth5[key][0]
            pred_label = 1 if ontCall[key][0] >= args.pred_threshold else 0
            y_truth.append(bs_label)
            y_pred.append(pred_label)
        ## Calculate precision and recall
        prec = precision_score(y_truth, y_pred)
        recall = recall_score(y_truth, y_pred)
        f1 = f1_score(y_truth, y_pred)
        accuracy = accuracy_score(y_truth, y_pred)
        logger.info(f"Joined report: chr={args.chr} (Meth={sum(y_truth):,}, Unmeth={len(y_truth)-sum(y_truth)}), precision={prec}, recall={recall}, f1={f1}, accuracy={accuracy}")
        print(f"Joined report\t{args.chr}\t{sum(y_truth)}\t{len(y_truth)-sum(y_truth)}\t{prec}\t{recall}\t{f1}\t{accuracy}\n")

        y_truth = []
        y_pred = []
        for key in bgTruth5:
            bs_label = bgTruth5[key][0]
            if key in ontCall:
                pred_label = 1 if ontCall[key][0] >= args.pred_threshold else 0
            else:
                pred_label = 0
            y_truth.append(bs_label)
            y_pred.append(pred_label)
        ## Calculate precision and recall
        prec = precision_score(y_truth, y_pred)
        recall = recall_score(y_truth, y_pred)
        f1 = f1_score(y_truth, y_pred)
        accuracy = accuracy_score(y_truth, y_pred)
        logger.info(f"BS-seq report: chr={args.chr} (Meth={sum(y_truth):,}, Unmeth={len(y_truth)-sum(y_truth):,}), precision={prec}, recall={recall}, f1={f1}, accuracy={accuracy}")
        print(
            f"Bs-seq report\t{args.chr}\t{sum(y_truth)}\t{len(y_truth) - sum(y_truth)}\t{prec}\t{recall}\t{f1}\t{accuracy}\n")
        sys.exit(0)

    ## Evalutate on chromsome
    logger.info(f'\n\n####################\n\n')
    logger.info(f'Evalutate on chr={args.chr}')
    chrSet = [args.chr]
    # Combine one/two replicates using DeepMod methods
    bgTruth1, ret1 = combineBGTruthList_by_DeepModPaper(bgTruthList, covCutoff=1, filterChrs=chrSet)
    bgTruth5 = {key: bgTruth1[key] for key in bgTruth1 if bgTruth1[key][1] >= 5}
    bgTruth10 = {key: bgTruth1[key] for key in bgTruth1 if bgTruth1[key][1] >= 10}

    ## Load and evalutate on calls
    dataset = []
    for callstr in args.calls:
        callencode, callfn = callstr.split(':')
        if len(callfn) == 0:
            continue
        if callencode in ['DeepMod.C', 'DeepMod.Cluster', 'Guppy']:
            ontCall = import_call(callfn, callencode, baseFormat=baseFormat, enable_cache=enable_cache, using_cache=using_cache, include_score=False, siteLevel=True)
        else:
            ontCall_ReadLevel = import_call(callfn, callencode, baseFormat=baseFormat, enable_cache=enable_cache, using_cache=using_cache, include_score=False, siteLevel=False)
            ontCall = readLevelToSiteLevelWithCov(ontCall_ReadLevel, minCov=1, toolname=callencode)

        ret = report_performance_deepmod(ontCall, bgTruth5, threshold=args.pred_threshold)
        logger.info(f'ret={ret}')
        ret.update({"coverage": 5, "chr": args.chr, "tool": callencode})
        dataset.append(ret)

    df = pd.DataFrame(dataset)
    logger.info(df)

    outfn = os.path.join(out_dir, f'sanity.check.using.DeepMod.paper.performance.chr_{args.chr}.threshold_{args.pred_threshold:.2f}.csv')
    df.to_csv(outfn)
    logger.info(f'save to {outfn}')
    logger.info("### Sanity check DONE")



import argparse
import glob
import logging
import os
import subprocess
from collections import defaultdict
from datetime import datetime

import pandas as pd
from tqdm import tqdm

from nanocompare.global_config import set_log_debug_level, pic_base_dir, logger

## TODO: not use anymore


run_log_dir = '/projects/li-lab/Nanopore_compare/result/running-logs'
basedir = run_log_dir

tool_names = ['Tombo', 'DeepMod', 'DeepSignal', 'Nanopolish', 'Megalodon']
tool_list_on_sumner = ['Tombo', 'DeepMod', 'DeepSignal', 'Nanopolish']
tool_list_on_winter = ['Guppy', 'Megalodon']

ntarget_dict = {'HL60': 50, 'K562': 50, 'APL': 50, 'NA19240': 300}  # , 'NA19240': 300

pkldir = '/projects/li-lab/yang/results/share_prj/result/running-logs'
sunmer_pkl = os.path.join(pkldir, 'sumner.task.resource.summary.pkl')
winter_pkl = os.path.join(pkldir, 'winter.task.resource.summary.pkl')
batch_fast5_pkl = os.path.join(pkldir, 'dsname.batch.fast5.summary.pkl')


def get_jobid_and_taskid(fn):
    """
    Sample input file name bascal.Guppy.K562.N50.batch49.22818.err
    :param fn:
    :return:
    """
    fn = os.path.basename(fn)
    last1_index = fn.rindex('.')
    last2_index = fn[:last1_index].rindex('.')
    taskid = fn[last2_index + 1: last1_index]

    last3_index = fn[:last2_index].rindex('.')
    batchstr = fn[last3_index + 1:last2_index].replace('batch', '')

    return taskid, batchstr


def winter_task_summary():
    dataset = defaultdict(list)
    for dsname in ntarget_dict:
        logging.info(dsname)
        ## Resquiggle collection
        #   HL60-Runs/HL60-N50-basecall/log
        basecall_logdir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-N{ntarget_dict[dsname]}-basecall', 'log')
        pat_fns = os.path.join(basecall_logdir, 'bascal.*.out')
        fnlist = glob.glob(pat_fns)
        logging.info(f'Basecall collect: {len(fnlist)}')

        for fn in tqdm(fnlist):
            taskid, batchid = get_jobid_and_taskid(fn)
            command = f"""
                   seff {taskid}
                   """
            ret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")

            dataset['dsname'].append(dsname)
            dataset['batchid'].append(int(batchid))
            dataset['type'].append('basecall')
            dataset['job.results'].append(ret)

        ## Tool methcall collection
        for tool in tool_list_on_winter:
            methcall_logdir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-{tool}-N{ntarget_dict[dsname]}', f'{dsname}-{tool}-N{ntarget_dict[dsname]}-methcall', 'log')
            pat_fns = os.path.join(methcall_logdir, '*.mcal.*.batch*.*.out')
            fnlist = glob.glob(pat_fns)
            logging.info(f'Methcall of {tool} collect: {len(fnlist)}')

            for fn in tqdm(fnlist):
                taskid, batchid = get_jobid_and_taskid(fn)
                # logging.debug(taskid)
                command = f"""
        seff {taskid}
        """
                ret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")
                dataset['dsname'].append(dsname)
                dataset['batchid'].append(int(batchid))
                dataset['type'].append(tool)
                dataset['job.results'].append(ret)
    df = pd.DataFrame.from_dict(dataset)
    outfn = os.path.join(pic_base_dir, 'winter.task.resource.summary.pkl')
    df.to_pickle(outfn)
    logging.info(f'save to {outfn}')

    outfn = os.path.join(pic_base_dir, 'winter.task.resource.summary.xlsx')
    df.to_excel(outfn)


def winter_task_summary_na19240():
    dsname = 'NA19240'
    dataset = defaultdict(list)
    logging.info(dsname)
    ## Resquiggle collection
    #   HL60-Runs/HL60-N50-basecall/log
    basecall_logdir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-N{ntarget_dict[dsname]}-basecall', 'log')
    pat_fns = os.path.join(basecall_logdir, 'bascal.*.out')
    fnlist = glob.glob(pat_fns)
    logging.info(f'Basecall collect: {len(fnlist)}')

    for fn in tqdm(fnlist):
        taskid, batchid = get_jobid_and_taskid(fn)
        command = f"""
               seff {taskid}
               """
        ret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")

        rt_secs, mem_gb, jobid, array_jobid = str_extract_time_mem(ret)

        if not rt_secs:  # Failed tasks are not consider
            continue

        dataset['dsname'].append(dsname)
        dataset['tool'].append('basecall')
        dataset['batchid'].append(int(batchid))
        dataset['running.time.seconds'].append(rt_secs)
        dataset['mem.usage.gb'].append(mem_gb)
        dataset['jobid'].append(jobid)
        dataset['array.jobid'].append(array_jobid)
        dataset['job.results'].append(ret)

    ## Tool methcall collection
    for tool in tool_list_on_winter:
        methcall_logdir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-{tool}-N{ntarget_dict[dsname]}', f'{dsname}-{tool}-N{ntarget_dict[dsname]}-methcall', 'log')
        pat_fns = os.path.join(methcall_logdir, '*.mcal.*.batch*.*.out')
        fnlist = glob.glob(pat_fns)
        logging.info(f'Methcall of {tool} collect: {len(fnlist)}')

        for fn in tqdm(fnlist):
            taskid, batchid = get_jobid_and_taskid(fn)
            # logging.debug(taskid)
            command = f"""
    seff {taskid}
    """
            ret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")
            rt_secs, mem_gb, jobid, array_jobid = str_extract_time_mem(ret)

            if not rt_secs:  # Failed tasks are not consider
                continue

            dataset['dsname'].append(dsname)
            dataset['tool'].append(tool)
            dataset['batchid'].append(int(batchid))
            dataset['running.time.seconds'].append(rt_secs)
            dataset['mem.usage.gb'].append(mem_gb)
            dataset['jobid'].append(jobid)
            dataset['array.jobid'].append(array_jobid)
            dataset['job.results'].append(ret)

    df = pd.DataFrame.from_dict(dataset)
    outfn = os.path.join(pic_base_dir, 'na19240.winter.task.resource.summary.pkl')
    df.to_pickle(outfn)
    logging.info(f'save to {outfn}')

    outfn = os.path.join(pic_base_dir, 'na19240.winter.task.resource.summary.xlsx')
    df.to_excel(outfn)


def winter_megalodon_task_summary():
    dataset = defaultdict(list)
    for dsname in ntarget_dict:
        logging.info(dsname)
        methdir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-Megalodon-N{ntarget_dict[dsname]}', f'{dsname}-Megalodon-N{ntarget_dict[dsname]}-methcall', 'log')

        pat_fns = os.path.join(methdir, '*.mcal.*.out')
        fnlist = glob.glob(pat_fns)
        logging.info(f'Megalodon of {dsname} collect: {len(fnlist)}')

        for fn in tqdm(fnlist):
            taskid, batchid = get_jobid_and_taskid(fn)
            command = f"""
                               seff {taskid}
                               """
            jobret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")

            dataset['dsname'].append(dsname)
            dataset['batchid'].append(int(batchid))
            dataset['tool'].append('Megalodon')
            dataset['job.results'].append(jobret)

            runtime_seconds, mem_gb, jobid, array_jobid = str_extract_time_mem(jobret)
            dataset['running.time.seconds'].append(runtime_seconds)
            dataset['mem.usage.gb'].append(mem_gb)
            dataset['jobid'].append(jobid)
            dataset['array.jobid'].append(array_jobid)
        df1 = pd.DataFrame.from_dict(dataset)
        df2 = pd.read_pickle(batch_fast5_pkl)
        df = df1.merge(df2, on=['dsname', 'batchid'], how='left')

        dfout = df[['dsname', 'tool', 'batchid', 'jobid', 'array.jobid', 'fast5', 'running.time.seconds', 'mem.usage.gb', 'job.results']]
        outfn = os.path.join(pic_base_dir, 'recalculate.running.summary.Megalodon.xlsx')
        dfout.to_excel(outfn)

        dfout = df[['dsname', 'tool', 'batchid', 'jobid', 'array.jobid', 'fast5', 'running.time.seconds', 'mem.usage.gb']]
        outfn = os.path.join(pic_base_dir, 'recalculate.running.summary.Megalodon.csv')
        dfout.to_csv(outfn)


def sunmer_task_summary():
    dataset = defaultdict(list)
    for dsname in ntarget_dict:
        logging.info(dsname)
        ## Resquiggle collection
        #   HL60-Runs/HL60-N50-basecall/log
        basecall_logdir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-N{ntarget_dict[dsname]}-resquiggle', 'log')
        pat_fns = os.path.join(basecall_logdir, 'rsquigl.*.out')
        fnlist = glob.glob(pat_fns)
        logging.info(f'Resquiggle collect: {len(fnlist)}')

        for fn in tqdm(fnlist):
            taskid, batchid = get_jobid_and_taskid(fn)
            command = f"""
                seff {taskid}
                """
            ret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")

            dataset['dsname'].append(dsname)
            dataset['batchid'].append(int(batchid))
            dataset['type'].append('resquiggle')
            dataset['job.results'].append(ret)

        ## Tool methcall collection
        for tool in tool_list_on_sumner:
            # HL60-Runs/HL60-Nanopolish-N50/HL60-Nanopolish-N50-methcall/log
            methcall_logdir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-{tool}-N{ntarget_dict[dsname]}', f'{dsname}-{tool}-N{ntarget_dict[dsname]}-methcall', 'log')
            # logging.debug(meth_logdir)

            pat_fns = os.path.join(methcall_logdir, '*.mcal.*.batch*.*.out')
            fnlist = glob.glob(pat_fns)
            logging.info(f'Methcall of {tool} collect: {len(fnlist)}')

            for fn in tqdm(fnlist):
                taskid, batchid = get_jobid_and_taskid(fn)
                # logging.debug(taskid)
                command = f"""
    seff {taskid}
    """
                ret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")
                dataset['dsname'].append(dsname)
                dataset['batchid'].append(int(batchid))
                dataset['type'].append(tool)
                dataset['job.results'].append(ret)

    df = pd.DataFrame.from_dict(dataset)
    # logging.info(df)
    outfn = os.path.join(pic_base_dir, 'sumner.task.resource.summary.pkl')
    df.to_pickle(outfn)
    logging.info(f'save to {outfn}')
    outfn = os.path.join(pic_base_dir, 'sumner.task.resource.summary.xlsx')
    df.to_excel(outfn)


def sunmer_task_summary_na19240():
    dsname = 'NA19240'
    dataset = defaultdict(list)
    resquiggle_dir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-N{ntarget_dict[dsname]}-resquiggle', 'log')
    pat_fns = os.path.join(resquiggle_dir, 'rsquigl.*.out')
    logger.debug(f'pat_fns={pat_fns}')
    fnlist = glob.glob(pat_fns)
    logging.info(f'Resquiggle collect: {len(fnlist)}')

    for fn in tqdm(fnlist):
        taskid, batchid = get_jobid_and_taskid(fn)
        command = f"""
            seff {taskid}
            """
        ret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")

        rt_secs, mem_gb, jobid, array_jobid = str_extract_time_mem(ret)

        if not rt_secs:  # Failed tasks are not consider
            continue

        dataset['dsname'].append(dsname)
        dataset['tool'].append('resquiggle')
        dataset['batchid'].append(int(batchid))
        dataset['running.time.seconds'].append(rt_secs)
        dataset['mem.usage.gb'].append(mem_gb)
        dataset['jobid'].append(jobid)
        dataset['array.jobid'].append(array_jobid)
        dataset['job.results'].append(ret)

    ## Tool methcall collection
    for tool in tool_list_on_sumner:
        # HL60-Runs/HL60-Nanopolish-N50/HL60-Nanopolish-N50-methcall/log
        methcall_logdir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-{tool}-N{ntarget_dict[dsname]}', f'{dsname}-{tool}-N{ntarget_dict[dsname]}-methcall', 'log')
        # logging.debug(meth_logdir)

        pat_fns = os.path.join(methcall_logdir, '*.mcal.*.batch*.*.out')
        fnlist = glob.glob(pat_fns)
        logging.info(f'Methcall of {tool} collect: {len(fnlist)}')

        for fn in tqdm(fnlist):
            taskid, batchid = get_jobid_and_taskid(fn)
            # logging.debug(taskid)
            command = f"""
seff {taskid}
"""
            ret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")

            rt_secs, mem_gb, jobid, array_jobid = str_extract_time_mem(ret)

            if not rt_secs:  # Failed tasks are not consider
                continue

            dataset['dsname'].append(dsname)
            dataset['tool'].append(tool)
            dataset['batchid'].append(int(batchid))
            dataset['running.time.seconds'].append(rt_secs)
            dataset['mem.usage.gb'].append(mem_gb)
            dataset['jobid'].append(jobid)
            dataset['array.jobid'].append(array_jobid)
            dataset['job.results'].append(ret)

    df = pd.DataFrame.from_dict(dataset)
    # logging.info(df)
    outfn = os.path.join(pic_base_dir, 'na19240.sumner.task.resource.summary.pkl')
    df.to_pickle(outfn)
    logging.info(f'save to {outfn}')
    outfn = os.path.join(pic_base_dir, 'na19240.sumner.task.resource.summary.xlsx')
    df.to_excel(outfn)


def dataset_batch_summary():
    dataset = defaultdict(list)
    for dsname in ntarget_dict:
        logging.info(dsname)
        for batchid in range(1, ntarget_dict[dsname] + 1):
            septdir = os.path.join(basedir, f'{dsname}-Runs', f'{dsname}-N{ntarget_dict[dsname]}-sept', f'{batchid}')
            command = f"""
                           ls {septdir}/*.fast5 | wc -l
                            """
            ret = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE).stdout.read().decode("utf-8")
            logging.debug(f'batchid={batchid}, ret={ret}')
            dataset['dsname'].append(dsname)
            dataset['batchid'].append(batchid)
            dataset['fast5'].append(int(ret.strip()))

    df = pd.DataFrame.from_dict(dataset)
    outfn = os.path.join(pic_base_dir, 'dsname.batch.fast5.summary.pkl')
    df.to_pickle(outfn)
    logging.info(f'save to {outfn}')
    outfn = os.path.join(pic_base_dir, 'dsname.batch.fast5.summary.xlsx')
    df.to_excel(outfn)


def parse_time(time_string):
    start_time = datetime.strptime("00:00:00", '%H:%M:%S')
    try:
        end_time = datetime.strptime(time_string, '%H:%M:%S')
    except:
        end_time = datetime.strptime(time_string, '%d-%H:%M:%S')

    duration_time = end_time - start_time
    return duration_time
    pass


def str_extract_time_mem(jobret):
    """
    Example:

    Job ID: 23443
    Array Job ID: 23398_23
    Cluster: winter
    User/Group: liuya/jaxuser
    State: COMPLETED (exit code 0)
    Nodes: 1
    Cores per node: 16
    CPU Utilized: 00:24:01
    CPU Efficiency: 9.26% of 04:19:28 core-walltime
    Job Wall-clock time: 00:16:13
    Memory Utilized: 2.52 GB
    Memory Efficiency: 0.16% of 1.56 TB

    :param jobret:
    :return:
    """
    cpu_use_time = None
    wall_clock_time = None
    mem_use = None
    jobid = None
    array_jobid = None

    for line in jobret.splitlines():
        # if line.strip().startswith('State: F') or line.strip().startswith('State: CANCELLED'):
        #     return None, None, None, None

        if line.strip().startswith('Job ID:'):
            jobid = line.strip().replace('Job ID:', '').strip()

        if line.strip().startswith('Array Job ID:'):
            array_jobid = line.strip().replace('Array Job ID:', '').strip()

        if line.strip().startswith('CPU Utilized:'):
            cpu_use_time = line.strip().replace('CPU Utilized:', '').strip()
        elif line.strip().startswith('Memory Utilized:'):
            mem_use = line.strip().replace('Memory Utilized:', '').strip()
        elif line.strip().startswith('Job Wall-clock time:'):
            wall_clock_time = line.strip().replace('Job Wall-clock time:', '').strip()

    cpu_use_time = parse_time(cpu_use_time)
    wall_clock_time = parse_time(wall_clock_time)

    # 71.49 MB, we report GB
    if mem_use.endswith('MB'):
        mem_gb = float(mem_use.replace('MB', '').strip()) / 1000
    elif mem_use.endswith('GB'):
        mem_gb = float(mem_use.replace('GB', '').strip())
    else:
        raise Exception(f'Unrecognized mem={mem_use} from jobret={jobret}')

    return cpu_use_time.total_seconds(), wall_clock_time.total_seconds(), mem_gb, jobid


def running_resouce_extraction(row):
    """

    :param row:
    :return:
    """
    jobret = row['job.results']
    cpu_time, wall_clock_time, mem_gb, jobid = str_extract_time_mem(jobret)

    return pd.Series([cpu_time / 60 / 60, wall_clock_time / 60 / 60, mem_gb, jobid], index=['cpu.time', 'wall.clock.time', 'mem.usage', 'jobid'])


def unify_data_df():
    df1 = pd.read_pickle(winter_pkl)
    df2 = pd.read_pickle(sunmer_pkl)
    df3 = pd.read_pickle(batch_fast5_pkl)
    df = pd.concat([df1, df2])

    df = df.merge(df3, on=['dsname', 'batchid'], how='left')

    df[['running.time', 'mem.usage', 'running.time.seconds', 'mem.usage.gb']] = df.apply(running_resouce_extraction, axis=1)

    logger.info(df)

    logger.info(list(df.columns))

    run_report_columns = ['dsname', 'batchid', 'type', 'fast5', 'running.time', 'mem.usage', 'running.time.seconds', 'mem.usage.gb', 'job.results']
    outdf = df[run_report_columns]
    outfn = os.path.join(pic_base_dir, 'running.summary.table.xlsx')
    outdf.to_excel(outfn)
    logger.info(f'save to {outfn}')

    outdf = df[run_report_columns[:-1]]
    outfn = os.path.join(pic_base_dir, 'running.summary.table.csv')
    outdf.to_csv(outfn)
    logger.info(f'save to {outfn}')


def recalculate(fnlist=['na19240.sumner.task.resource.summary.pkl', 'na19240.winter.task.resource.summary.pkl']):
    dflist = []
    for fn in fnlist:
        dflist.append(pd.read_pickle(os.path.join(pkldir, fn)))
    df = pd.concat(dflist)

    df3 = pd.read_pickle(batch_fast5_pkl)
    df = df.merge(df3, on=['dsname', 'batchid'], how='left')
    logger.debug(df)

    dataset = defaultdict(list)
    for index, row in df.iterrows():
        if row['tool'] not in tool_list_on_sumner + tool_list_on_winter:
            continue
        dsname = row['dsname']
        batchid = row['batchid']
        runt = row['running.time.seconds']
        memg = row['mem.usage.gb']

        basecall_row = df[(df['dsname'] == dsname) & (df['batchid'] == batchid) & (df['tool'] == 'basecall')].iloc[0, :]
        resquiggle_row = df[(df['dsname'] == dsname) & (df['batchid'] == batchid) & (df['tool'] == 'resquiggle')].iloc[0, :]

        if row['tool'] in ['DeepSignal', 'Tombo']:
            runt += basecall_row['running.time.seconds'] + resquiggle_row['running.time.seconds']
            memg += basecall_row['mem.usage.gb'] + resquiggle_row['mem.usage.gb']
        elif row['tool'] in ['Nanopolish', 'DeepMod']:
            runt += basecall_row['running.time.seconds']
            memg += basecall_row['mem.usage.gb']
        dataset['dsname'].append(dsname)
        dataset['tool'].append(row['tool'])
        dataset['batchid'].append(row['batchid'])
        dataset['fast5'].append(row['fast5'])
        dataset['running.time.seconds'].append(runt)
        dataset['mem.usage.gb'].append(memg)
    outdf = pd.DataFrame.from_dict(dataset)
    logger.info(outdf)
    outfn = os.path.join(pic_base_dir, 'recalculate.running.summary.na19240.csv')
    outdf.to_csv(outfn)


def parse_arguments():
    """
    :return:
    """
    parser = argparse.ArgumentParser(description='Resource-summary')
    parser.add_argument('--cpu-task', action='store_true')
    parser.add_argument('--gpu-task', action='store_true')
    parser.add_argument('--dataset-batch', action='store_true')
    parser.add_argument('--unify', action='store_true')
    parser.add_argument('--recalculate', action='store_true')
    parser.add_argument('--megalodon', action='store_true')
    parser.add_argument('--na19240-winter', action='store_true')
    parser.add_argument('--na19240-sumner', action='store_true')
    parser.add_argument('--collect-data', action='store_true')

    return parser.parse_args()


def collect_log_data():
    fnlist = glob.glob(os.path.join(basedir, '*.summary*xlsx'))
    logger.info(fnlist)

    dflist = []
    for fn in fnlist:
        df = pd.read_excel(fn)
        df = df.rename(columns={'type': 'tool'})
        dflist.append(df)
    retdf = pd.concat(dflist)
    retdf = retdf[['dsname', 'tool', 'batchid', 'job.results']]
    outfn = os.path.join(pic_base_dir, 'running.logs.on.four.datasets.xlsx')
    retdf.to_excel(outfn)
    return retdf
    pass


if __name__ == '__main__':
    set_log_debug_level()
    args = parse_arguments()
    logger.debug(args)

    if args.cpu_task:
        sunmer_task_summary()
    if args.gpu_task:
        winter_task_summary()
    if args.dataset_batch:
        dataset_batch_summary()
    if args.unify:
        unify_data_df()
    if args.recalculate:
        recalculate()
    if args.megalodon:
        winter_megalodon_task_summary()
    if args.na19240_sumner:
        sunmer_task_summary_na19240()
    if args.na19240_winter:
        winter_task_summary_na19240()
    if args.collect_data:
        infn = os.path.join('/projects/li-lab/yang/results/2021-04-15', 'running.logs.on.four.datasets.full.logs.xlsx')
        df = pd.read_excel(infn, index_col=0)

        fast5fn = os.path.join(basedir, 'dsname.batch.fast5.table.xlsx')
        fast5df = pd.read_excel(fast5fn, index_col=0)

        outdf = df.merge(fast5df, on=['dsname', 'batchid'], how='inner')
        logger.info(outdf.iloc[0, :])

        outdf[['cpu.time', 'wall.clock.time', 'mem.usage', 'jobid']] = outdf.apply(running_resouce_extraction, axis=1)
        logger.info(df)

        outfn = os.path.join(pic_base_dir, 'running.logs.on.four.datasets.step1.extracted.fields.xlsx')
        outdf.to_excel(outfn)

        outdf = outdf.groupby(by=['dsname', 'tool']).agg({'cpu.time': 'sum', 'wall.clock.time': 'max', 'mem.usage': 'max'})

        # Recalculate the total time and mem usage
        # add basecall to DeepSignal, Tombo, DeepMod, and Nanopolish
        # add resquiggle to DeepSignal and Tombo
        # remove Guppy methcall

        outfn = os.path.join(pic_base_dir, 'running.logs.on.four.datasets.step2.before.calculation.xlsx')
        outdf.to_excel(outfn)

        for index, row in outdf.iterrows():
            if row.name[1] in ['DeepSignal', 'Tombo', 'DeepMod', 'Nanopolish']:
                # Get basecalled row
                basecallRow = outdf.loc[[(row.name[0], 'basecall')]].iloc[0, :]

                # row['cpu.time'] += basecallRow['cpu.time']
                # row['wall.clock.time'] += basecallRow['wall.clock.time']
                # row['mem.usage'] = max(row['mem.usage'], basecallRow['mem.usage'])
                outdf.at[index, 'cpu.time'] += basecallRow['cpu.time']
                outdf.at[index, 'wall.clock.time'] += basecallRow['wall.clock.time']
                outdf.at[index, 'mem.usage'] = max(outdf.at[index, 'mem.usage'], basecallRow['mem.usage'])

            if row.name[1] in ['DeepSignal', 'Tombo']:
                # Get resquiggle row
                resquiggleRow = outdf.loc[[(row.name[0], 'resquiggle')]].iloc[0, :]

                # row['cpu.time'] += resquiggleRow['cpu.time']
                # row['wall.clock.time'] += resquiggleRow['wall.clock.time']
                # row['mem.usage'] = max(row['mem.usage'], resquiggleRow['mem.usage'])
                outdf.at[index, 'cpu.time'] += resquiggleRow['cpu.time']
                outdf.at[index, 'wall.clock.time'] += resquiggleRow['wall.clock.time']
                outdf.at[index, 'mem.usage'] = max(outdf.at[index, 'mem.usage'], resquiggleRow['mem.usage'])

        outdf = outdf.rename(columns={"cpu.time": "CPU Utilized", 'wall.clock.time': "Job Wall-clock Time", "mem.usage": "Memory Utilized"})
        outdf = outdf.reset_index()
        outdf.loc[outdf.dsname == 'NA19240', 'Job Wall-clock Time'] = outdf.loc[outdf.dsname == 'NA19240', 'Job Wall-clock Time'] * 6
        outdf = outdf[outdf['tool'].isin(tool_names)]

        outfn = os.path.join(pic_base_dir, 'running.logs.on.four.datasets.step3.five.tools.xlsx')
        outdf.to_excel(outfn, index=False)
        logger.info(outdf)

    logger.info("DONE")



#!/usr/bin/env python3
# @Author   : Yang Liu
# @FileName : computeRawReadsCoverage.py
# @Software : NANOME project
# @Organization : JAX Li Lab
# @Website  : https://github.com/TheJacksonLaboratory/nanome

"""
Calculate nanopore raw fast5 reads coverage at each CpG sites based on genomic context for Table S1.
"""
import argparse
import glob
import gzip
import os
import re
from multiprocessing import Pool

import pandas as pd
import pybedtools
from pybedtools import BedTool
from tqdm import tqdm

from nanocompare.eval_common import get_dna_seq_from_reference, open_file_gz_or_txt, find_bed_filename, get_ref_fasta, \
    get_region_bed, intersect_bed_regions
from nanocompare.global_config import set_log_debug_level, logger, pic_base_dir, temp_dir, data_base_dir
from nanocompare.global_settings import humanChrSet, location_filename_to_abbvname, \
    datasets_order, narrowCoordNameList, cg_density_coord_name_list, \
    rep_coord_name_list, referenceGenomeFile, nanome_version

# used for convert region bed cov to base level cov
rawReadDir = '/pod/2/li-lab/Nanopore_compare/data/Nanopore_cov'
rawReadDir = '/projects/li-lab/Nanopore_compare/nanome_paper_result/basecall_results/raw_fast5_reads_cpg_coverage'

# base level cov bed files (combined + - strand)
base_cov_dir = '/projects/li-lab/Nanopore_compare/nanome_paper_result/basecall_results/raw_fast5_reads_cpg_coverage'

# dir to newly concordant and discordant perf results bed dir
bedDir = '/projects/li-lab/yang/results/2021-07-08'

def convert_region_to_cpg_base(dsname):
    """
    Conver bed file of regions into CpG base bed files

    Assume infn is 0-based file, we print 1-based output. Because 1-based site level reports are easy to stats overlap with region files.
    :param infn:
    :return:
    """

    fnlist = glob.glob(os.path.join(rawReadDir, f'{dsname}*coverage.*strand.bed.gz'))
    logger.info(f'convert_region_to_cpg_base:{fnlist}')

    outfn = os.path.join(pic_base_dir, f'{dsname}.rawfast5.coverage.base.bed.gz')
    outfile = gzip.open(outfn, 'wt')

    print_first = True
    for infn in fnlist:
        logger.info(f'processing file={infn}')
        infile = open_file_gz_or_txt(infn)
        for row in tqdm(infile):
            tmp = row.strip().split(" ")
            if print_first:
                print_first = False
                logger.info(f'row={row}, tmp={tmp}')
            chr = tmp[0]
            start = int(tmp[1])
            end = int(tmp[2])
            cov = int(tmp[3])
            # TODO: get strand info, + strand link to CG's C, - strand link to CG's G
            strand = tmp[4]

            if chr not in humanChrSet:  # filter out non-human chrs
                continue

            # we want get seq at least two bases, evaluate 'CG' patterns
            # if end == start + 1:
            #     newend = end + 1
            # else:
            #     newend = end
            newend = end + 1

            dnastr = get_dna_seq_from_reference(chr, start, newend, ref_fasta=refFasta)
            for cpg in re.finditer("CG", dnastr):
                cpgstart = start + cpg.start()
                if strand == '+':  # point to CG's C, just report that position (0-based) into 1-based
                    outstart = cpgstart + 1
                elif strand == '-':  # negative strand, point to CG's G, based on seq of + strand
                    outstart = cpgstart + 2
                else:
                    raise Exception(f'strand={strand} is no suppport')

                out_txt = '\t'.join([chr, str(outstart), str(outstart), str(cov), '.', strand])
                outfile.write(f'{out_txt}\n')
                pass
        infile.close()
    outfile.close()


def preprocess_bed_to_cgp_base():
    dsname_list = ['HL60', 'K562', 'APL', 'NA19240']
    dsname_list = ['NA12878']

    for dsname in dsname_list:
        logger.info(f'dsname={dsname}')
        convert_region_to_cpg_base(dsname)


def count_sites_in_coord(readBed, coordfn, tagname, cutoff_list):
    """
    Counts how many CpG sites from readBed are also in range of coordinate file

    Sample readBed files:
    head HL60.rawfast5.coverage.base.bed
    chr1	10470	10470	8	.	-
    chr1	10472	10472	6	.	-
    chr1	10485	10485	4	.	-
    chr1	10490	10490	6	.	-

    :param readBed:
    :param coordfn:
    :param cutoff_list:
    :return:
    """
    ret = {}

    coordBed = get_region_bed(coordfn)
    intersectBed = intersect_bed_regions(readBed, coordBed, coordfn)

    covList = []
    for row in intersectBed:
        cov = int(row[3])
        covList.append(cov)
    covSeries = pd.Series(covList)
    for i, cutoff in enumerate(cutoff_list):
        ret.update({f'{tagname}.cutoff{cutoff}': (covSeries >= cutoff).sum()})

    return ret


def get_len_of_bedtool(bed_obj, cov_col=3, cutoff=1):
    """
    Get the number of rows (sites) of BedTool object
    :param bed_obj:
    :param cutoff:
    :return:
    """
    # covList = []
    # for row in bed_obj:
    #     covList.append(int(row[cov_col]))
    # covSeries = pd.Series(covList)
    df = bed_obj.to_dataframe()
    return (df.iloc[:, cov_col] >= cutoff).sum()


def save_dataset(dataset, tagname=""):
    ## dataframe for raw reads at each genomic regions
    df = pd.DataFrame(dataset)
    logger.info(df)

    column_list = ['dsname', 'total', 'total_cutoff'] + \
                  ["Singletons", "Non-singletons", "Promoters", "Exons", "Introns", "Intergenic", "CpG islands",
                   "CpG shores", "CpG shelves"] + \
                  ["CG_20", "CG_40", "CG_60", "CG_80", "CG_100"] + \
                  ["rep_SINE", "rep_LINE", "rep_LTR", "rep_DNA", "rep_Others"] + \
                  ['Concordant', 'Discordant', 'filename']

    for cutoff in cutoff_list:
        outdf = pd.concat([df.loc[:, ['dsname', 'filename', 'total']], df.filter(regex=f'.cutoff{cutoff}$', axis=1)],
                          axis=1)

        oldname_list = outdf.columns
        newname_list = [the_name.replace(f'.cutoff{cutoff}', "") for the_name in oldname_list]
        outdf.columns = newname_list

        # outfn = os.path.join(pic_base_dir,
        #                      f'raw.fast5.reads.cpg.coverage.across.regions.cutoff{cutoff}.not.sorted{tagname}.xlsx')
        # outdf.to_excel(outfn)

        ## Reorder columns based on list, if not exist, skit it
        selcols = [colname for colname in column_list if colname in outdf.columns]
        outdf = outdf[selcols]

        outdf['dsname'] = pd.Categorical(df['dsname'], datasets_order)
        outdf = outdf.sort_values(by='dsname', ascending=True)
        outfn = os.path.join(pic_base_dir,
                             f'raw.fast5.reads.cpg.coverage.across.regions.cutoff{cutoff}{tagname}.table.s1.xlsx')
        outdf.to_excel(outfn)
        logger.info(f'save to {outfn}')


def combine_na12878_coverage_bed():
    baseDir = "/fastscratch/liuya/nanocompare/NA12878-coverage"
    outfn = os.path.join(pic_base_dir, "NA12878-allChrs.coverage.bothstrand.bed.gz")
    outf = gzip.open(outfn, 'wt')
    for chrName in humanChrSet:
        logger.info(f"Processing chr={chrName}")
        flist = glob.glob(os.path.join(baseDir, f"NA12878-{chrName.upper()}.coverage.*.bed.gz"))
        logger.info(flist)

        for fn in flist:
            logger.info(fn)
            with gzip.open(fn, 'rt') as inf:
                for row in tqdm(inf):
                    tmp = row.strip().split(" ")
                    chr = tmp[0]
                    if chr != chrName:
                        continue
                    outf.write(f"{row}")

    outf.close()
    logger.info(f"save to {outfn}")


def report_raw_fast5_cpg_in_regions_table():
    """
    Generate Figure 2 C and D data: stats #cpgs of raw fast5 sequencing in each region
    :return:
    """
    dataset = []
    # distribution_dataset = defaultdict(list)
    for dsname in dsname_list:
        fnlist = glob.glob(os.path.join(args.base_cov_dir, f'{dsname}.rawfast5.coverage.base.bed.gz'))
        if len(fnlist) != 1:
            raise Exception(f"Not only one file, fnlist={fnlist}, for dsname={dsname}")

        fn = fnlist[0]
        logger.info(fn)

        rawReadBed = BedTool(fn).sort()
        logger.info(f'len(rawReadBed)={len(rawReadBed):,}')
        dataDict = {'dsname': dsname, 'filename': fn, 'total': len(rawReadBed)}

        # list of return for each parrallellized jobs
        retList = []

        ## we now use to_dataframe() for BedTool
        bed_df_cov_col = rawReadBed.to_dataframe().iloc[:, 3]

        ret1 = {}
        for cutoff in cutoff_list:
            ret1.update({f'total_cutoff.cutoff{cutoff}': (bed_df_cov_col >= cutoff).sum()})

        logger.info(f"Evaluated regions: {eval_regions}")
        with Pool(processes=16) as pool:
            for bedfn in eval_regions:
                tagname = location_filename_to_abbvname[os.path.basename(bedfn)]
                ret = pool.apply_async(count_sites_in_coord, (rawReadBed, bedfn, tagname,),
                                       kwds={'cutoff_list': cutoff_list})
                retList.append(ret)

            concordantFileName = find_bed_filename(basedir=bedDir,
                                                   pattern=f'{dsname}*hg38_nonsingletons.concordant.bed')
            ret = pool.apply_async(count_sites_in_coord, (rawReadBed, concordantFileName, 'Concordant',),
                                   kwds={'cutoff_list': cutoff_list})
            retList.append(ret)

            discordantFileName = find_bed_filename(basedir=bedDir,
                                                   pattern=f'{dsname}*hg38_nonsingletons.discordant.bed')
            ret = pool.apply_async(count_sites_in_coord, (rawReadBed, discordantFileName, 'Discordant',),
                                   kwds={'cutoff_list': cutoff_list})
            retList.append(ret)

            pool.close()
            pool.join()
        # Get each jobs's return results
        retList = [ret.get() for ret in retList]

        # Update returned results into dataDict: each line of results in the table
        dataDict.update(ret1)
        for ret in retList:
            dataDict.update(ret)
        dataset.append(dataDict)
        logger.debug(dataDict)

        ## Temp output for real-time view
        save_dataset(dataset, tagname='tmp')

    save_dataset(dataset)


def parse_arguments():
    parser = argparse.ArgumentParser(prog='comp_raw_read_cov (NANOME)',description='Plot and export data for Nanocompare paper.')
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s v{nanome_version}')
    parser.add_argument("cmd", help="name of command")
    parser.add_argument('--bedtools-tmp', type=str, help='bedtools temp dir', default=temp_dir)
    parser.add_argument('--genome-annotation', type=str, help='genome annotation dir',
                        default=os.path.join(data_base_dir, 'genome-annotation'))
    parser.add_argument('--reference-genome', type=str, help='reference genome file',
                        default=referenceGenomeFile)
    parser.add_argument('--base-cov-dir', type=str, help='raw fast5 base coverage dir',
                        default=base_cov_dir)
    parser.add_argument('--bed-dir', type=str, help='bed dir for concordant and discordant',
                        default=bedDir)
    args = parser.parse_args()
    return args


if __name__ == '__main__':
    set_log_debug_level()
    args = parse_arguments()
    logger.debug(args)

    os.makedirs(args.bedtools_tmp, exist_ok=True)
    pybedtools.helpers.set_tempdir(args.bedtools_tmp)

    refFasta = None

    # Combine NA12878 all chrs bed regions
    if args.cmd == 'combine-na12878':
        combine_na12878_coverage_bed()

    # Generate bed sites file from region file using sequencing string
    if args.cmd == 'region-to-base':
        refFasta = get_ref_fasta(ref_fn=args.reference_genome)
        preprocess_bed_to_cgp_base()

    # Calculate raw fast5 cpg sites in each region
    if args.cmd == 'report-raw':
        dsname_list = ['HL60', 'K562', 'APL', 'NA19240', 'NA12878']
        # Nanopore reads coverage cutoff
        cutoff_list = [3]

        # list of evaluated regions (except for Genome-wide)
        # eval_regions = narrowCoordFileList[1:] + cg_density_file_list + rep_file_list

        eval_regions = [os.path.join(args.genome_annotation, cofn) for cofn in narrowCoordNameList[1:]] + \
                       [os.path.join(args.genome_annotation, cofn) for cofn in
                        cg_density_coord_name_list] + \
                       [os.path.join(args.genome_annotation, cofn) for cofn in rep_coord_name_list]

        report_raw_fast5_cpg_in_regions_table()
    logger.info("### computeRawCoverage DONE")



#!/usr/bin/env python3

import os, sys, time
from collections import defaultdict
import glob

import numpy as np

from scipy import stats

import locale
##locale.setlocale(locale.LC_ALL, 'en_US') ## failed in google cloud

import tensorflow as tf

batch_size = 4096

cov_thrd = 5

def readBed(bedfile, t_chr=None, t_start=None, t_end=None):
   print('read {}'.format(bedfile)); sys.stdout.flush()
   beddict = defaultdict()
   with open(bedfile, 'r') as bedreader:
      start_time = time.time();
      line = bedreader.readline();
      while True:
         line = bedreader.readline();
         if not line: break;

         line = line.strip();
         if len(line)>20:
            mchr, start_pos, end_pos, _, _, m_strand, _, _, _, true_cov, meth_perc = line.split()
            start_pos = int(start_pos)
            true_cov = int(true_cov)
            if true_cov < cov_thrd: continue;
            meth_perc = round(int(meth_perc)/100.0, 3)
            if (t_chr not in [None, mchr]) or (not ((t_start==None or start_pos>=t_start) and (t_end==None or start_pos<=t_end))):
                 continue;
            if true_cov==0: continue
            beddict[(mchr, m_strand, start_pos)] = meth_perc
   return beddict

def readpredmod(predmodf, preddict, t_chr=None, t_start=None, t_end=None, cgCposdict=None):
   print('read {}'.format(predmodf)); sys.stdout.flush()
   with open(predmodf, 'r') as mr:
      while True:
          line = mr.readline();
          if not line: break;
          line = line.strip();
          if len(line)>0:
             lsp = line.split();
             cur_chr = lsp[0];
             cur_pos = int(lsp[1]);
             cur_strand = lsp[5];

             if not (cgCposdict==None or (cur_chr, cur_strand, cur_pos) in cgCposdict): continue

             cur_cov = int(lsp[9]);
             cur_m_p = int(lsp[10]);
             cur_m_c = int(lsp[11]);
             if (t_chr not in [None, cur_chr]) or (not ((t_start==None or cur_pos>=t_start) and (t_end==None or cur_pos<=t_end))):
                continue;
             if cur_cov==0: continue;
 
             if (cur_chr, cur_strand, cur_pos) not in preddict:
                 preddict[(cur_chr, cur_strand, cur_pos)] = [cur_cov, round(cur_m_p/100.0, 3), cur_m_c, line]
             else:
                 print("Warning_duplicate {}".format(predmodf))
                 preddict[(cur_chr, cur_strand, cur_pos)][0] += cur_cov
                 preddict[(cur_chr, cur_strand, cur_pos)][2] += cur_m_c
                 if preddict[(cur_chr, cur_strand, cur_pos)][0]>0:
                    preddict[(cur_chr, cur_strand, cur_pos)][1] = round(preddict[(cur_chr, cur_strand, cur_pos)][2]/float(preddict[(cur_chr, cur_strand, cur_pos)][0]), 3)



pred_file = sys.argv[1]+'.%s.C.bed'
save_file = sys.argv[1]+'_clusterCpG.%s.C.bed'
gmotfolder = sys.argv[2]

mpat = 'Cg'; mposinpat=0
stposofinterest = None; edposofinterest = None;

nbsize = 25;
#train_mod = '/projects/li-lab/yang/tools/latest-version/DeepMod/train_mod/na12878_cluster_train_mod-keep_prob0.7-nb25-chr1/{}.cov{}.nb{}'.format(mpat, cov_thrd, nbsize)
train_mod = sys.argv[3]

chrkeys = []
for i in range(1, 23):
   chrkeys.append("chr%d" % i)
chrkeys.append("chrX")
chrkeys.append("chrY")
chrkeys.append("chrM")


new_saver = tf.train.import_meta_graph(train_mod+'.meta')
print(new_saver); sys.stdout.flush()
with tf.Session() as sess:
   print("restore model: {} {}".format(train_mod+'.meta', train_mod[:train_mod.rindex('/')+1]))
   print(new_saver.restore(sess,tf.train.latest_checkpoint(train_mod[:train_mod.rindex('/')+1]))); sys.stdout.flush()

   mgraph = tf.get_default_graph()
   output = mgraph.get_tensor_by_name('output:0')
   X = mgraph.get_tensor_by_name('X:0')
   keep_prob = mgraph.get_tensor_by_name('keep_prob:0')
 
   for chrofinterest in chrkeys:
      #read pred
      preddict = defaultdict()

      cur_cg_pos = '%s/motif_%s_C.bed' % (gmotfolder, chrofinterest)
      if not os.path.isfile(cur_cg_pos): 
         print("Warning_motif!!! no file {}".format(cur_cg_pos))
         continue;
      if not os.path.isfile(pred_file % chrofinterest):
         print("Warning_pred!!! no file {}".format(pred_file % chrofinterest))
         continue;
 
      cgposdict = defaultdict();
      with open(cur_cg_pos, 'r') as mr:
         while True:
            line = mr.readline();
            if not line: break;
            lsp = line.split();
            cgposdict[ (lsp[0], lsp[2], int(lsp[1]) ) ] = True
      print("{}: read {} done! ".format(len(cgposdict), cur_cg_pos)); sys.stdout.flush()
      readpredmod(pred_file % chrofinterest, preddict, chrofinterest, cgCposdict=cgposdict)
      print("size={} vs ".format(len(preddict), len(cgposdict) )); sys.stdout.flush()

      train_data = []
      pdkeys = sorted(list( preddict.keys() ))
      for cspk in pdkeys: # preddict:
         if cspk not in cgposdict: 
            print("not in cpg warning!!! {} {}".format(chrofinterest, cspk))

         partner_pos = (cspk[0], '-' if cspk[1]=='+' else '+', cspk[2]+1 if cspk[1]=='+' else cspk[2]-1)
         cur_x = [preddict[cspk][1], preddict[partner_pos][1] if partner_pos in preddict else 0]
         for pdis in range(11):
            cur_x.append(0)
         cur_x.append(0)
         if len(train_data)<10: print("test")
         for rpos in range(cspk[2]-nbsize, cspk[2]+nbsize+1):
            if rpos in [cspk[2], partner_pos[2]]: continue;
            
            if (cspk[0], '+', rpos) in cgposdict and (cspk[0], '+', rpos) in preddict:
                cur_x[int(preddict[(cspk[0], '+', rpos)][1]/0.1+0.5) + 3] += 1
                cur_x[2] += 1
                if len(train_data)<10: print("\t\t{}: {}".format((cspk[0], '+', rpos), preddict[(cspk[0], '+', rpos)]))
            elif (cspk[0], '-', rpos) in cgposdict and (cspk[0], '-', rpos) in preddict:
                cur_x[int(preddict[(cspk[0], '-', rpos)][1]/0.1+0.5) + 3] += 1
                cur_x[2] += 1
                if len(train_data)<10: print("\t\t{}: {}".format((cspk[0], '-', rpos), preddict[(cspk[0], '-', rpos)]))
         for i in range(3, len(cur_x)):
            if cur_x[2]>0: cur_x[i] = round(cur_x[i]/float(cur_x[2]), 3)
         if len(train_data)<10: print('\t{}'.format(cur_x)); sys.stdout.flush()
         train_data.append(cur_x)

      try:
         print("format data: data={}; {}".format(len(train_data), len(train_data[0]))); sys.stdout.flush()
      except:
         pass
      
      batch_data = np.array_split(train_data, int(len(train_data)/batch_size) if len(train_data)>batch_size else 2)
      m_pred_new_per = []
      for i in range(len(batch_data)):
         try:
             moutp = sess.run([output], feed_dict={X:batch_data[i], keep_prob:1})
             for mpind in moutp:
                 for curpd in mpind:
                    m_pred_new_per.append(curpd)
         except:
            pass
      try:
         print("new per: {}, {}  {} {}".format(len(pdkeys), len(train_data), len(m_pred_new_per), curpd ))
      except:
         pass
      for wind in range(10):
         try:
            print("'{}' <{}> {}".format(preddict[pdkeys[wind]][-1], m_pred_new_per[wind], train_data[wind]))
         except:
            pass
      with open(save_file % chrofinterest, 'w') as mwriter:
         for wind in range(len(pdkeys)):
            mwriter.write("{} {}\n".format(preddict[pdkeys[wind]][-1], int(m_pred_new_per[wind]*100)))
 
 



#!/usr/bin/env python3

"""
This tool is to clean old basecall analyses for NA12878, etc.
"""
import argparse
import glob
import os.path
from multiprocessing import Pool

import numpy as np
import h5py

# Group name need to be cleaned in fast5 files
groupName = "Analyses"


def parse_arguments():
    parser = argparse.ArgumentParser(description='Clean old basecall info in fast5 files.')
    parser.add_argument('-i', nargs='+', help='list of input fast5 files', default=[])
    parser.add_argument('--processor', type=int, help='number of processors/threads', default=8)
    parser.add_argument('--verbose', action='store_true', help="True if print debug info")
    parser.add_argument('--is-indir', action='store_true', help="True if input is folder")

    args = parser.parse_args()
    return args


def clean_filelist(fnlist):
    """
    Clean for a filelist, will write to them
    :param fnlist:
    :return:
    """
    cntClean = 0
    for fn in fnlist:
        try:
            with h5py.File(fn, 'r+') as handle:  # ref: https://docs.h5py.org/en/stable/high/file.html?highlight=h5py.File#h5py.File
                # print(f"Detect old {groupName}, we clean it.")
                if groupName in list(handle.keys()):  # clean if found any group named 'Analyses'
                    del handle[groupName]
                    cntClean += 1
        except: ## avoid corrupted fast5 files
            pass
    return cntClean


if __name__ == '__main__':
    args = parse_arguments()
    # print(args)
    cntClean = 0

    if args.is_indir:
        fnlist = []
        for bdir in args.i:
            fnlist += glob.glob(os.path.join(bdir, "*.fast5"))
        pass
    else:
        fnlist = args.i

    if args.verbose or args.is_indir:
        print(f"Total fast5 files: {len(fnlist)}")

    # Split to multiple processing
    fnlistArgs = np.array_split(fnlist, args.processor)
    with Pool(processes=args.processor) as pool:
        ret = pool.map(clean_filelist, fnlistArgs)
    totalCleanedFiles = sum(ret)

    if args.verbose or args.is_indir:
        print(f"Total cleaned files: {totalCleanedFiles}")
